# ----------------------------------------------------------------------------
#
#  Copyright (C) 2013-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


from zita_jacktools.jackclient import JackClient
from zita_jacktools import jacknoise_ext


class JackNoise(JackClient) :
    """
    Generate white and pink noise. There can be up to 100
    independent (uncorrelated) outputs.
    """

    OFF   = 0
    WHITE = 1
    PINK  = 2

    
    def __init__(self, nchan, client_name, server_name = None):
        """
        Create a new JackNoise instance with 'nchan' independent outputs.

        The result should be checked using get_state().
        The optional 'server_name' allows to select between running
        Jack servers.
        """
        self._jnoise, base = jacknoise_ext.makecaps (self, client_name, server_name, nchan)
        super().__init__(base)


    def set_output (self, chan, type, level):     
        """
        Set noise type and RMS level in dB FS. Initially all channels are OFF.
        """
        return jacknoise_ext.set_output (self._jnoise, chan, type, level)





#!/usr/bin/python

# ----------------------------------------------------------------------------
#
#  Copyright (C) 2018-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


import numpy as np
import matplotlib.pyplot as plt
from zita_jacktools.jacklr4filt import JackLR4filt
from zita_jacktools.jacksignal import JackSignal


# Measure frequency response of Linkwitz-Riley filters
#

# Create and connect objects.
#
J = JackSignal("JackSignal")
if J.get_state() < 0:
    print ("Failed to create JackSignal -- is Jack running ?")
    exit(1)
J.create_output (0, "out")
J.create_input (0, "in-0")
J.create_input (1, "in-1")
J.create_input (2, "in-2")
J.silence()
F = JackLR4filt (1, 3, 0.1, "JackLR4filt")

J.connect_output (0, "JackLR4filt:in_0")
J.connect_input (0, "JackLR4filt:out_0")
J.connect_input (1, "JackLR4filt:out_1")
J.connect_input (2, "JackLR4filt:out_2")


# Get Jack info.
#
name, fsamp, period = J.get_jack_info ()

# Parameters.
#
impval = 1.0
fftlen = 16 * 1024

# Generate data.
#
Aout = np.zeros ([100,], dtype = np.float32)
Aout [0] = impval
Ahp = np.empty ([fftlen,], dtype = np.float32)
Alp = np.empty ([fftlen,], dtype = np.float32)
Abp = np.empty ([fftlen,], dtype = np.float32)
Fax = np.linspace (0, fsamp / 2, num = fftlen // 2 + 1)

J.set_output_data (0, Aout)
J.set_input_data (0, Ahp, nskip = period)
J.set_input_data (1, Alp, nskip = period)
J.set_input_data (2, Abp, nskip = period)

F.set_filter (0, 0, 500, -6.0, 0, 0)   # HP, 500 Hz 
F.set_filter (0, 1, 0, 0, 500, -6.0)   # LP, 500 Hz
F.set_filter (0, 2, 50, -3, 200, -3.0) # BP, 50...200 Hz

# Some experiments.
#F.set_gain (0, 0.316) # -10 dB
#F.set_delay (0, 0.01) # 10 ms

# Measure.
#
J.process()
J.wait()
del J, F

Fhp = np.fft.rfft (Ahp)
Flp = np.fft.rfft (Alp)
Fbp = np.fft.rfft (Abp)

# Plot result.
#
fig = plt.figure (figsize=(8,5), facecolor='white')
fig.canvas.manager.set_window_title ('oct1band')
ax1 = fig.add_axes ([0.06, 0.06, 0.90, 0.90])
ax1.set_xlim (1e1, 24e3)
ax1.set_ylim (-50, 5)
ax1.set_xscale ('log')
ax1.grid ()
ax1.plot (Fax, 20 * np.log10 (np.abs (Fhp) + 1e-10), color='b', lw=1)
ax1.plot (Fax, 20 * np.log10 (np.abs (Flp) + 1e-10), color='g', lw=1)
ax1.plot (Fax, 20 * np.log10 (np.abs (Fhp + Flp) + 1e-10), color='r', lw=1)
ax1.plot (Fax, 20 * np.log10 (np.abs (Fbp) + 1e-10), color='k', lw=1)
plt.show()




# ----------------------------------------------------------------------------
#
#  Copyright (C) 2018-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


from PyQt6 import QtGui


# Global definitions.
#
# Note: other modules should use 'import globdef as',
# and not 'from globdef import'. The latter will
# ignore changes made after import, e.g. by darkpal().
 

# Sizes.
#
ELM_XS = 41
ELM_YS = 41
IP_YS = 93
OP_XS = 93

# Fonts.
#
VAL_FONT = QtGui.QFont ("sans", 7, QtGui.QFont.Weight.Bold)
BUT_FONT = QtGui.QFont ("sans", 9, QtGui.QFont.Weight.Bold)
LAB_FONT = QtGui.QFont ("sans", 8, QtGui.QFont.Weight.Bold)

# Callback actions.
#
ACT_GAIN  = 0
ACT_DELAY = 1
ACT_MUTE  = 2
ACT_SOLO  = 3
#ACT_CLEAR = 4
#ACT_COPY  = 5
#ACT_PASTE = 6

# Colors.
#
COL0 = QtGui.QColor (0, 0, 0)
COL2 = QtGui.QColor (255, 0, 0)
COL3 = QtGui.QColor (255, 140, 0)
COL4 = QtGui.QColor (255, 235, 0)
COL5 = QtGui.QColor (0, 255, 0)
COL6 = QtGui.QColor (0, 0, 150)
COL9 = QtGui.QColor (255, 255, 255)
FG_N = QtGui.QColor (210, 210, 220)
MK_N = COL0

# Dark theme.
#
def darkpal (app):
    global FG_N, MK_N
    
    bgc = QtGui.QColor (70, 70, 70)
    fgc = QtGui.QColor (255, 255, 255)
    dark = QtGui.QColor (0, 0, 0)
    lite = QtGui.QColor (150, 150, 150)
    pal = app.palette()
    pal.setColor(pal.ColorRole.Window, bgc)
    pal.setColor(pal.ColorRole.Button, bgc)
    pal.setColor(pal.ColorRole.Base, bgc)
    pal.setColor(pal.ColorRole.WindowText, fgc)
    pal.setColor(pal.ColorRole.ButtonText, fgc)
    pal.setColor(pal.ColorRole.Text, fgc)
    pal.setColor(pal.ColorRole.Dark, dark)
    pal.setColor(pal.ColorRole.Light, lite)
    app.setPalette (pal)
    FG_N = QtGui.QColor (80, 80, 100)
    MK_N = COL9




# ----------------------------------------------------------------------------
#
#  Copyright (C) 2018-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


from zita_jacktools.jackclient import JackClient
from zita_jacktools import jacklr4filt_ext


class JackLR4filt(JackClient):

    """
    Fourth order Linkwitz-Riley filters. Typical use is for speaker
    crossover filters. Each output can be lowpass, highpass, or both
   (bandpass), with variable gain and delay.
    """

    def __init__(self, ninp, nout, maxdel, client_name, server_name = None):
        """
        Create a new JackLR4 instance with given number of inputs
        and outputs, and maximum delay. 

        The result should be checked using get_state().
        The maximum number of inputs and outputs is 100.
        The maximum delay is limited to 1 second.

        The optional 'server_name' allows to select between running
        Jack servers.
        """
        self._jlr4filt, base = jacklr4filt_ext.makecaps (self, client_name, server_name, ninp, nout, maxdel)
        super().__init__(base)


    def set_filter (self, inp, out, hpf, hps, lpf, lps) :
        """
        Set input, HP and/or LP filter parameters for given output.

        'hpf', 'lpf' : HP and LP cutoff frequency, or 0 to disable a filter.
        'hps', 'lps' : filter shape. This sets the gain (dB) at the cutoff
         frequency. The valid range is from -3 (Butterworth) to -6 (Linkwitz-Riley).
        """
        return jacklr4filt_ext.set_filter (self._jlr4filt, inp, out, hpf, hps, lpf, lps)


    def set_delay (self, out, delay) :
        """
        Set output delay in seconds.
        """
        return jacklr4filt_ext.set_delay (self._jlr4filt, out, delay)


    def set_gain (self, out, gain) :
        """
        Set output gain.

        The gain value is linear, not in dB.
        """
        return jacklr4filt_ext.set_gain (self._jlr4filt, out, gain)





#!/usr/bin/python

# ----------------------------------------------------------------------------
#
#  Copyright (C) 2008-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


import sys
import numpy as np
from time import sleep
from math import *
from zita_jacktools.jacksignal import JackSignal
from zita_audiotools.audiofile import write_audio
sys.path.append ('..')
from utils.logsweep import genlogsweep


# ------- Impulse response measurement using log sweep  ---------
#
# Make sure zita-rev1 is running, output full wet.


# Create a JackSignal object and connect.
#
J = JackSignal("JackSignal")
if J.get_state() < 0:
    print ("Failed to create JackSignal -- is Jack running ?")
    exit(1)
J.create_output (0, "out")
J.create_input (0, "in.L")
J.create_input (1, "in.R")
J.silence()
J.connect_output (0, "zita-rev1:in.L")
J.connect_output (0, "zita-rev1:in.R")
J.connect_input (0, "zita-rev1:out.L")
J.connect_input (1, "zita-rev1:out.R")


# Get Jack info.
#
name, fsamp, period = J.get_jack_info ()

# Parameters.
#
fmin = 20.0
fmax = 23e3
amp = 0.1
tfade1 = 0.01
tsweep = 1.0
tfade2 = 0.01
timpulse = 3.0

# Generate sweep.
#
Afwd, Ainv, pcorr = genlogsweep (amp, fmin, fmax, fsamp, tfade1, tsweep, tfade2)

# Create input buffer.
#
k = int (fsamp * timpulse + Afwd.shape [0] + 0.5)
Ainp = np.empty ([k,2], dtype = np.float32)

# Connect buffer to ports.
#
J.set_output_data (0, Afwd)
J.set_input_data (0, Ainp [:,0])
J.set_input_data (1, Ainp [:,1])

# Run the test
#
J.process()
J.wait ()
del J

# Process and store result
#
AL = np.convolve (Ainp [:,0], Ainv, mode = 'valid')
AR = np.convolve (Ainp [:,1], Ainv, mode = 'valid')
write_audio (AL, "imp-L.wav")
write_audio (AR, "imp-R.wav")




#!/usr/bin/python

import numpy as np
from math import *
from time import sleep
from zita_jacktools.jackgainctl import JackGainctl
from zita_jacktools.jacksignal import JackSignal
import matplotlib.pyplot as plt


# Use JackSignal to test the 'dezippering' of JackGainctl.
# This will diplay the output signal on both a linear and
# dB scale. Zoom in to measure the times.


S = JackSignal("S")
if S.get_state() < 0:
    print ("Failed to create JackSignal -- is Jack running ?")
    exit(1)
S.create_output (0, "out-1")
S.create_input (0, "in-1")
S.silence()

G = JackGainctl (1, 'G')
S.connect_output (0, "G:in_0")
S.connect_input  (0, "G:out_0")


name, fsamp, period = S.get_jack_info ()
fsamp = 48000
Lout = fsamp // 10
Linp = fsamp
Aout = np.zeros ((Lout,), dtype = np.float32)
Ainp = np.zeros ((Linp,), dtype = np.float32)

T = np.linspace (0, 1, Linp, endpoint = False)

w = 2e3 * 2 * pi / fsamp
for i in range (Lout): Aout [i] = sin (w * i)

S.set_output_data (0, Aout, nloop = 11)
S.set_input_data (0, Ainp, nskip = period)
S.process()
sleep (0.1)

# Off to -20 dB, linear fade-in in 10 ms.
G.set_gain (-20.0)
sleep (0.15)

# -20 to 0 dB, this should take three 10 ms
# segments of +6 dB, plus one segment of +2 dB,
# for a total of 40 ms.
G.set_gain (0.0)
sleep (0.15)

# Mute, linear fade-out in 10 ms.
G.set_mute (1)
sleep (0.15)

# Unmute, linear fade-in in 10 ms.
G.set_mute (0)
sleep (0.15)

# 60 dB gain change, 100 ms.
G.set_gain (-60)
sleep (0.15)

# 90 dB gain change, 10 ms because
# target value is equivalent to 'off'.
G.set_gain (-150)
S.wait()


del S
del G

X1 = np.abs (Ainp)
X2 = 20 * np.log10 (X1 + 1e-20)

fig = plt.figure (figsize=(8,6), facecolor='white')
ax1 = fig.add_axes ([0.07, 0.54, 0.9, 0.44])
ax2 = fig.add_axes ([0.07, 0.05, 0.9, 0.44])
ax1.grid ()
ax2.grid ()
ax1.set_ylim (0, 1.2)
ax2.set_ylim (-120, 10)
ax1.plot (T, X1, color='g', lw=1)
ax2.plot (T, X2, color='b', lw=1)
plt.show()



# ----------------------------------------------------------------------------
#
#  Copyright (C) 2018-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


import array

class IoElm ():
    def __init__(self):
        self.reset ()
        
    def reset (self):
        self.gain = 0.0
        self.mute = 0
        
        
class IoVec ():
    def __init__ (self, nelm):
        self.nelm = nelm
        self.elms = [ IoElm () for i in range (nelm) ]

    def reset (self):
        for E in self.elms: E.reset ()

    def elem (self, elm):
        return self.elms [elm]

    def write (self, fobj):
        n = len (self.elms)
        for i in range (self.nelm):
            E = self.elms [i]
            f = ''
            if E.mute: f += 'M'
            if (E.gain == 0.0) and (f == ''): continue
            fobj.write ("%3d %6.1f %s\n" % (i, E.gain, f))

    def read (self, s):
        try:
            E = self.elms [int (s [0])]
            E.gain = float (s [1])
            if len (s) > 2:
                E.mute = 'M' in s [2]
        except:
            return

        
class MatElm ():
    def __init__ (self):
        self.reset ()

    def reset (self):
        self.gain = -200.0
        self.delay = 0.0
        self.inv = 0

        
class Matrix ():
    def __init__ (self, ninp, nout):
        n = ninp * nout
        self.ninp = ninp
        self.nout = nout
        self.elms = [ MatElm () for i in range (n) ]

    def reset (self):
        for E in self.elms: E.reset ()

    def elem (self, inp, out):
        return self.elms [out * self.ninp + inp]
        
    def write (self, fobj):
        for out in range (self.nout):
            for inp in range (self.ninp):
                E = self.elem (inp, out)
                f = ''
                if E.inv: f += 'I'
                if (E.gain <= -200.0) and (E.delay == 0) and (f == ''):
                    continue
                fobj.write ("%3d %3d %8.1f %5.1f %s\n"
                            % (inp, out, E.gain, E.delay, f))
            
    def read (self, s):
        try:
            E = self.elem (int (s [0]), int (s [1]))
            E.gain = float (s [2])
            E.delay = float (s [3])
            if len (s) > 4:
                E.inv = 'I' in s [4]
        except:
            return
                
                    
class Matrixstate ():

    def __init__ (self, ninp, nout):
        self.inpgain = IoVec (ninp)
        self.outgain = IoVec (nout)
        self.matrix = Matrix (ninp, nout)
        
    def reset (self):
        self.inpgain.reset ()
        self.outgain.reset ()
        self.matrix.reset ()
        
    def write (self, fobj):
        fobj.write ("INP\n")
        self.inpgain.write (fobj)
        fobj.write ("OUT\n")
        self.outgain.write (fobj)
        fobj.write ("MAT\n")
        self.matrix.write (fobj)
        fobj.write ("END\n")

    def read (self, fobj):
        self.reset ()
        sect = None
        while True:
            s = fobj.readline ()
            if len (s) == 0: return
            s = s.split ()
            if len (s) == 0: continue
            if   s [0] == 'INP': sect = self.inpgain
            elif s [0] == 'OUT': sect = self.outgain
            elif s [0] == 'MAT': sect = self.matrix
            elif s [0] == 'END': return
            elif sect is not None: sect.read (s)
        



#!/usr/bin/python

# ----------------------------------------------------------------------------
#
#  Copyright (C) 2008-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------

import sys
import numpy as np
import matplotlib.pyplot as plt
from scipy.signal import convolve # much faster than numpy.convolve
from zita_jacktools.jacksignal import JackSignal
sys.path.append ('..')
from utils.logsweep import genlogsweep


# ------- Frequency response using log sweep, deconvolution and fft ---------
#
# This method can be used to obtain the impulse response or frequency
# response in noisy conditions, e.g. for measuring a room or a speaker.
# This works because a sweep has much more energy than a single sample
# as used in freqresp1.py. To see this work you could repeat this test
# with the pink noise output of japa connected to jnoisemeter. The noise
# level is -20 dB, the same as the sweep used here, so the S/N ratio is
# as bad as 0 dB. Still the measured impulse will be quite clean.
#
# An important thing to understand is that the impulse or frequency
# response you get with this method will be that of the system being
# tested in series with a bandpass filter. The shape of that filter
# is determined by the sweep parameters, and the filter will be a
# linear-phase one. That means there will some response even in
# negative time - before any output should appear at all. To get
# the correct frequency response, this must be included in the
# input to the Fourier transform. In this test we use 50 ms.
#
#
# Make sure jnoisemeter is running, using
# input 1, and select the A, C or ITU filter.


# Create a JackSignal object and connect.
#
J = JackSignal("JackSignal")
if J.get_state() < 0:
    print ("Failed to create JackSignal -- is Jack running ?")
    exit(1)
J.create_output (0, "out")
J.create_input (0, "in")
J.silence()
J.connect_output (0, "jnoisemeter:in_1")
J.connect_input (0, "jnoisemeter:out")

# Get Jack info.
#
name, fsamp, period = J.get_jack_info ()

# Sweep parameters.
#
amp = 0.1
fmin = 10
fmax = 22e3
tfade1 = 0.25
tsweep = 8.00
tfade2 = 0.05

# FFT size
#
LFFT = 0x4000
NBIN = LFFT // 2 + 1

# How many samples before impulse to keep.
# Should be <= LFFT / 2.
#
OFFS = fsamp // 20  # 50 ms

# Generate sweep.
#
Afwd, Ainv, p = genlogsweep (amp, fmin, fmax, fsamp, tfade1, tsweep, tfade2)

# Create input buffer.
#
Ainp = np.zeros ([Afwd.shape [0] + LFFT,], dtype = np.float32)

# Connect buffer to ports.
#
J.set_output_data (0, Afwd)
J.set_input_data (0, Ainp, nskip = period)

# Run the test
#
print ("Measuring....")
J.process()
J.wait()
del J

# Deconvolve sweep and keep LFFT samples.
#
print ("Convolving....")
k = Ainv.shape [0] - 1 - OFFS
T = convolve (Ainp, Ainv, mode = 'full')
T = T [k:k + LFFT]
F = np.fft.rfft (T, LFFT)

# Display impulse and magnitude response.
#
print ("Display...")

# Time axis in samples or seconds.
Tax = np.linspace (0, LFFT, LFFT, endpoint = False) - OFFS
#Tax /= fsamp

fig = plt.figure (figsize=(8,6), facecolor='white')
ax1 = fig.add_axes ([0.07, 0.05, 0.90, 0.44])
ax1.set_ylim (-1.5, 1.5)
ax1.plot (Tax, T, color='b', lw=1)
ax1.grid ()

# Frequency axis in Hz.
Fax = np.linspace (0, fsamp / 2, NBIN, endpoint = True)

ax2 = fig.add_axes ([0.07, 0.53, 0.90, 0.44])
ax2.set_xlim (20, 20e3)
ax2.set_ylim (-60, 15)
ax2.set_xscale ('log')
ax2.plot (Fax, 20 * np.log10 (np.abs (F) + 1e-10), color='b', lw=1)
ax2.grid ()
plt.show()



# ----------------------------------------------------------------------------
#
#  Copyright (C) 2008-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


from array import array
from zita_jacktools.jackclient import JackClient
from zita_jacktools import jackkmeter_ext


class JackKmeter(JackClient) :

    """
    DSP part of the 'K' audio level meter for up to 100 channels.
    """

    def __init__(self, nchan, client_name, server_name = None):
        """
        Create a new JackKmeter instance.

        The result should be checked using get_state().
        The optional 'server_name' allows to select between running
        Jack servers.
        """
        self._rms = array ('f', (0 for i in range (nchan)))
        self._peak = array ('f', (0 for i in range (nchan)))
        self._jkmeter, base = jackkmeter_ext.makecaps (self, client_name, server_name, nchan, self._rms, self._peak)
        super().__init__(base)


    def get_levels(self) :
        """
        Return state and updated 'rms' and 'peak' arrays.
        """
        return jackkmeter_ext.get_levels (self._jkmeter), self._rms, self._peak





# ----------------------------------------------------------------------------
#
#  Copyright (C) 2013-2020 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


import numpy as np
from math import floor, log, exp, sin, cos, pi


def genlogsweep (amp, fmin, fmax, fsamp, tfi, tsw, tfo):

    """
    Generate logarithmic sweep, inverse sweep and phase correction.
   
    Parameters:
       amp:           Amplitude of sweep
       fmin, fmax:    Frequency range of the constant amplitude part. 
       fsamp:         Sample frequency.
       tsw:           Duration of constant amplitude part, seconds.
       tfi, tfo:      Fade in and out times, seconds.
   
    During the fade in and out segments the frequency increases at
    the same logarithmic rate as during the constant amplitude part
    from fmin to fmax. The fade out time should be chosen so that
    the frequency never exceeds half the sample rate.
   
    The inverse sweep amplitude is dimensioned such that after
    convolution the result is a bandlimited Dirac impulse with
    unity spectral magnitude between fmin and fmax.

    A rising frequency log sweep can be used to measure harmonic
    distortion as a function of frequency. This will appear in
    negative time after convolution with the inverse sweep.
    The phase correction can be used to calculate the exact phase
    of any harmonic. Contact the author if you need the details.
    """
    
    k0 = int (tfi * fsamp + 0.5)
    k1 = int (tsw * fsamp + 0.5)
    k2 = int (tfo * fsamp + 0.5)
    a = log (fmax / fmin) / k1
    b = fmin / (a * fsamp)
    r = 4.0 * a * a / amp
    n = k0 + k1 + k2
    Afwd = np.empty ([n,], dtype = np.float32)
    Ainv = np.empty ([n,], dtype = np.float32)
    for i in range (n):
        j = n - i - 1
        if    i < k0: g = sin (0.5 * pi * i / k0)
        elif  j < k2: g = sin (0.5 * pi * j / k2)
        else:         g = 1.0 
        d = b * exp (a * (i - k0))
        p = d - b
        p -= floor (p)
        x = g * sin (2 * pi * p)
        Afwd [i] = x * amp
        Ainv [j] = x * d * r
    return Afwd, Ainv, b - floor (b)





#!/usr/bin/python

# ----------------------------------------------------------------------------
#
#  Copyright (C) 2008-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


import numpy as np
from math import *
from time import sleep
from zita_jacktools.jacksignal import JackSignal


# ----------------------- 1 kHz bursts to test jkmeter --------------------------
#
# Make sure jkmeter is running.

# Create a JackSignal object with 1 output.
#
J = JackSignal("JackSignal")
if J.get_state() < 0:
    print ("Failed to create JackSignal -- is Jack running ?")
    exit(1)

J.create_output (0, "out_1")
J.silence()
J.connect_output (0, "jkmeter:in-1")

# Get Jack info.
#
name, fsamp, period = J.get_jack_info ()

# Create output buffer.
#
size = int (0.5 * fsamp + 0.1)
A = np.empty ([size,], dtype = np.float32)

# Generate 1 kHz sine wave
#
w = 1e3 * 2 * pi / fsamp
for i in range (size):
    A [i] = sin (w * i)

# Generate burst of size t in seconds.
#
def burst (t):
    print ("t = %5.3f" % (t,))
    n = int (t * fsamp + 0.5)
    if n > size: n = size
    J.set_output_data (0, A [0:n])
    J.process()
    J.wait ()

# Run the test.
#
tburst = [0.001, 0.002, 0.005, 0.01, 0.02, 0.05, 0.1, 0.2, 0.5]
twait = 5.0 # Let jkmeter fall back.
for t in tburst:
    burst (t)
    sleep (twait)



# ----------------------------------------------------------------------------
#
#  Copyright (C) 2008-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


from zita_jacktools import jackcontrol_ext


class JackControl():
    """
    Jack connection and transport control.
    """

    ACTIVE = 1
    FAILED = -1
    ZOMBIE = -2


    def __init__(self, client_name, server_name = None):
        """
        Create a new JackControl instance.

        The result should be checked using get_client_state().
        The optional 'server_name' allows to select between running
        servers.
        """
        self._capsule = jackcontrol_ext.create (self, client_name, server_name)


    def get_jack_info(self):
        """
        Return the tuple (jack client name, sample rate, period size).
        """
        return jackcontrol_ext.get_jack_info (self._capsule)


    def get_client_state(self):
        """
        Returns the current client state, which will be one of

        ACTIVE  = 1   The object is active.
        FAILED  = -1  Failed to become a Jack client.
        ZOMBIE  = -2  Kicked out by Jack.

        The only option in the last two states is to delete the object.
        """
        return jackcontrol_ext.get_client_state (self._capsule)


    def get_transport_state(self):
        """
        Returns the tuple (transport state, transport frame).
        
        Transport state is one of

        STOPPED = 0  Transport is stopped.
        PLAYING = 1  Transport is rolling.
        SYNCING = 2  Waiting for clients to locate.
        """
        return jackcontrol_ext.get_transport_state (self._capsule)


    def transport_start(self):
        """
        Start the jack transport.
        """  
        return jackcontrol_ext.transport_start(self._capsule)


    def transport_stop(self):
        """
        Stop the jack transport.
        """  
        return jackcontrol_ext.transport_stop((self._capsule,))


    def transport_locate(self, frame):
        """
        Locate jack transport to postion 'frame'.
        """  
        return jackcontrol_ext.transport_locate(self._capsule, frame)


    def connect_ports(self, srce, dest):
        """
        Connect readable jack port 'srce' to writeable jack port 'dest'
        """        
        return jackcontrol_ext.connect_ports(self._capsule, srce, dest)


    def disconn_ports(self, srce, dest):
        """
        Disconnect writeable jack port 'dest' from readable jack port 'srce'
        """        
        return jackcontrol_ext.disconn_ports(self._capsule, srce, dest)




#!/usr/bin/python

# ----------------------------------------------------------------------------
#
#  Copyright (C) 2008-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


import numpy as np
import matplotlib.pyplot as plt
from math import *
from time import sleep
from zita_jacktools.jacksignal import JackSignal


# ------- Frequency response measurement using impulse and FFT  ---------
#
# Make sure jnoisemeter is running, using
# input 1, and select the A, C or ITU filter.

# Create a JackSignal object and connect.
#
J = JackSignal("JackSignal")
if J.get_state() < 0:
    print ("Failed to create JackSignal -- is Jack running ?")
    exit(1)
J.create_output (0, "out-1")
J.create_input (0, "in-1")

J.silence()
J.connect_output (0, "jnoisemeter:in_1")
J.connect_input (0, "jnoisemeter:out")


# Get Jack info.
#
name, fsamp, period = J.get_jack_info ()

# Parameters
#
impval = 1.0
fftlen = 64 * 1024

# Generate data.
#
Aout = np.zeros ([100,], dtype = np.float32)
Aout [0] = impval

Ain1 = np.empty ([fftlen,], dtype = np.float32)
Freq = np.linspace (0, fsamp / 2, num = fftlen // 2 + 1)

J.set_output_data (0, Aout)
J.set_input_data (0, Ain1, nskip = period) # Skip one period.

# Run the test
#
J.process()
J.wait ()
del J

# Process the result
#
Spec = np.fft.rfft (Ain1)

# Display impulse and magnitude response.
#
fig = plt.figure (figsize=(8,6), facecolor='white')
ax1 = fig.add_axes ([0.07, 0.04, 0.90, 0.44])
ax1.set_ylim (-1.5, 1.5)
ax1.plot (Ain1 [0:100], color='b', lw=1)
ax1.grid ()
ax2 = fig.add_axes ([0.07, 0.53, 0.90, 0.44])
ax2.set_xlim (20, 20e3)
ax2.set_ylim (-60, 15)
ax2.set_xscale ('log')
ax2.plot (Freq, 20 * np.log10 (np.abs (Spec) + 1e-10), color='b', lw=1)
ax2.grid ()
plt.show()



#!/usr/bin/python

# ----------------------------------------------------------------------------
#
#  Copyright (C) 2013-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


import sys
import matplotlib.pyplot as plt
import numpy as np
from time import sleep
from math import hypot, log10
from zita_jacktools.jacksignal import JackSignal
sys.path.append ('..')
from utils.sinewave import *


# --------------------- IEC Intermodulation measurement ----------------------
#
# Run jack_nonlin with arguments 0.01 0.01 0


# We need a linear X axis for the bargraph.
#
LogF = np.linspace (2.7, 4.3, 17, endpoint = True)
Freq = np.power (10.0, LogF) # 500 Hz to 20 kHz, 1/3 oct.
Fdiff = 60
Level = -20
Mtime = 1.0

# Create a Jacksignal object.
#
J = JackSignal("IMtest")
if J.get_state() < 0:
    print ("Failed to create JackSignal -- is the server running ?")
    exit(1)

# Get Jack info.
#
name, Fsamp, period = J.get_jack_info()

# Create ports and connect
#
J.create_output (0, "out")
J.create_input (0, "in")
J.silence()
J.connect_output (0, "jack_nonlin:in")
J.connect_input (0, "jack_nonlin:out")

# Length of test signals in samples.
#
siglen = int (Mtime * Fsamp + 0.5)
margin = 1000
buflen = siglen + margin

# Input buffer.
#
Ainp = np.zeros ((buflen,), dtype = np.float32)


# Run test.
#
N = Freq.shape [0]
D2 = N * [0]
D3 = N * [0]
for i in range (N):

    # Generate test signal, will be scaled for output.
    #
    Fcent = Freq [i]
    Flo = Fcent - Fdiff / 2
    Fhi = Fcent + Fdiff / 2
    amp = pow (10.0, Level / 20.0)
    Aout = (  gen_sinewave (amp, Flo, Fsamp, buflen)
            + gen_sinewave (amp, Fhi, Fsamp, buflen)).astype (np.float32)

    # Generate reference signals for selective measurement.
    #
    Mlo = gen_complex (Flo, Fsamp, siglen)
    Mhi = gen_complex (Fhi, Fsamp, siglen)
    M3a = gen_complex (2 * Flo - Fhi, Fsamp, siglen)
    M3b = gen_complex (2 * Fhi - Flo, Fsamp, siglen)
    M2  = gen_complex (Fdiff, Fsamp, siglen)

    # Define signal buffers and run test. 
    #
    J.set_output_data (0, Aout)
    J.set_input_data (0, Ainp) 
    J.process()
    J.wait()
    
    # Skip margin samples and measure.
    # We are not interested in the phase here.
    #
    T = Ainp [margin:]
    Llo,p = sigdetect (T, Mlo)
    Lhi,p = sigdetect (T, Mhi)
    L2,p  = sigdetect (T, M2)
    L3a,p = sigdetect (T, M3a)
    L3b,p = sigdetect (T, M3b)

    # Print results.
    #
    dblo = 20 * log10 (Llo + 1e-20)
    dbhi = 20 * log10 (Lhi + 1e-20)
    D2 [i] = im2 = 100 * L2 / (Llo + Lhi)
    D3 [i] = im3 = 100 * hypot (L3b, L3b) / (Llo + Lhi)
    print ("Freq = %7.1f    Al = %5.1f  Ah = %5.1f   im2 = %6.3f%%  im3 = %6.3f%%" % (Fcent, dblo, dbhi, im2, im3))

# Stop the Jacksignal process.
del J

# Create a nice graph to present the results.
#
fig = plt.figure (figsize=(8,6), facecolor='white')
ax = fig.add_axes ([0.07, 0.06, 0.86, 0.88])
ax.set_title ("% Intermodulation distortion (IEC)")
ax.set_xlim (2.6, 4.4)
ax.set_xticks ((2.7, 3.0, 3.3, 3.7, 4.0, 4.3))
ax.set_xticklabels (('500', '1k', '2k', '5k', '10k', '20k'))
ax.set_ylim (1e-3, 1e1)
ax.set_yscale ('log')
ax.bar (LogF - 0.022, D2, 0.02, color = 'b')
ax.bar (LogF + 0.002, D3, 0.02, color = 'g')
ax.text (2.75, 5.0, 'IM2', size = 17, color = 'b')
ax.text (2.95, 5.0, 'IM3', size = 17, color = 'g')
ax.grid ()
plt.show ()
    
    



# ----------------------------------------------------------------------------
#
#  Copyright (C) 2018-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


import globdef as G
from PyQt6 import QtGui, QtCore, QtWidgets


# Base class for the two matrix windows.
#
class Matrixwin (QtWidgets.QWidget):
    
    def __init__(self, parent, nrow, ncol): 
        super (Matrixwin, self).__init__(parent)
        self.nrow = nrow
        self.ncol = ncol
        self.xs = ncol * (G.ELM_XS + 1) + 1
        self.ys = nrow * (G.ELM_YS + 1) + 1
        self.resize (self.xs, self.ys)


    def paintEvent(self, E):
        qp = QtGui.QPainter(self)
        qp.setRenderHint (QtGui.QPainter.RenderHint.Antialiasing)
        qp.setBrush (QtCore.Qt.BrushStyle.NoBrush)
        qp.setPen (QtGui.QPen (QtGui.QColor (G.MK_N), 0.2))
        qp.translate (0.5, 0.5)
        x = 0
        for i in range (self.ncol + 1):
            qp.drawLine (x, 0, x, self.ys)
            x += G.ELM_XS + 1
        y = 0
        for i in range (self.nrow + 1):
            qp.drawLine (0, y, self.xs, y)
            y += G.ELM_YS + 1
    



# ----------------------------------------------------------------------------
#
#  Copyright (C) 2018-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


import globdef as G
from PyQt5 import QtGui, QtCore, QtWidgets
from matrixwin import *
from controls import *


class Gainelm (QtWidgets.QWidget):

    def __init__(self, parent, style, callb, index): 
        super (Gainelm, self).__init__(parent)
        self.resize (G.ELM_XS, G.ELM_YS)
        self.callb = callb
        self.index = index
        self.invert = 0
        R = Rgainctl (self, style)
        R.move (7, 2)
        R.valueEvent.connect (self.value_event)
        R.pressEvent.connect (self.press_event)
        R.show ()
        self.gainctl = R
        L = QtWidgets.QLabel (self)
        L.setFont (G.VAL_FONT)
        L.setText ('')
        L.setGeometry (0, 30, G.ELM_XS - 8, 14)
        L.setAlignment (QtCore.Qt.AlignRight)
        self.gaintxt = L

    def redraw (self):
        gaindb = self.gainctl.get_value ()
        if (gaindb > -200):
            self.gaintxt.setText('%5.1f' % (gaindb,))    
            if self.invert: self.gainctl.set_state (2)
            else:           self.gainctl.set_state (1)
        else:
            self.gaintxt.setText('')    
            self.gainctl.set_state (0)    

    def value_event (self, args):
        self.redraw ()
        self.callb (self.index, G.ACT_GAIN, args.get_value (), self.invert)     

    def press_event (self, args):
        b = args.mousebutt
        if b == 2:
            self.invert ^= 1
            self.value_event (args)
            
    def get_state (self):
        return self.gainctl.get_value (), self.invert

    def set_state (self, gaindb, invert):
        self.gainctl.set_value (gaindb)
        self.invert = invert
        self.redraw ()
         
     
         
class Gainmatwin (Matrixwin):

    def __init__(self, parent, nrow, ncol, callb): 
        super (Gainmatwin, self).__init__(parent, nrow, ncol)
        pal = self.palette () 
        bgc = pal.color (pal.Window)
        style = RotaryStyle (11, bgc, [G.FG_N,G.COL4,G.COL3], [G.MK_N,G.COL0,G.COL0])
        self.controls = {}
        y = 1
        for r in range (nrow):
            x = 1
            for c in range (ncol):
                index = (r, c)
                E = Gainelm (self, style, callb, index)  
                E.move (x, y)
                self.controls [index] = E
                x += G.ELM_XS + 1
            y += G.ELM_YS + 1

    def get_state (self, row, col):
        return self.controls [(row, col)].get_state ()
    
    def set_state (self, row, col, gaindb, invert):
        self.controls [(row, col)].set_state (gaindb, invert)
        




# ----------------------------------------------------------------------------
#
#  Copyright (C) 2008-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


from zita_jacktools.jackclient import JackClient
from zita_jacktools import jackparameq_ext


class JackParameq(JackClient):

    """
    Multichannel equaliser.

    There can be up to eight sections, each of them can be a low or high
    shelf filter or a parametric one.
    The shelf filters have a variable shape. For the  parametric filter
    three variations are available, these differ only in the way the gain
    and bandwidth settings interact. See the 'shelfeq.py' and 'parameq.py'
    example programs for details.
    All controls are fully dezippered. Also switching bypass on/off will
    be click-free. Sections set to 0 dB gain are automatically bypassed.
    """

    def __init__(self, nchan, types, client_name, server_name = None):
        """
        Create a new JackParameq instance with 'nchan' channels.

        The result should be checked using get_state().
        The 'types' argument is a string of section types. Valid types
        are 'L' or 'H' for a low or high shelf filter, '1','2' or '3'
        for a parametric.
        The optional 'server_name' allows to select between running
        Jack servers.
        """
        self._jparameq, base = jackparameq_ext.makecaps (self, client_name,
                                                         server_name,
                                                         nchan, types)
        super().__init__(base)


    def set_filter (self, sect, freq, gain, shape):
        """
        Set filter parameters for single section.

        For a parametric 'shape' controls the relative bandwidth
        (1/Q), in the range 0.1 to 10. For the shelf filters the
        range is 0 to 1. The 'shelfeq.py' examples programs shows
        how this affects the shape of the response. Frequencies
        are in Hz, gains in dB.
        """
        return jackparameq_ext.set_filter (self._jparameq, sect,
                                           freq, gain, shape)

    
    def set_gain (self, gain):
        """
        Set makeup gain in dB.
        """
        return jackparameq_ext.set_gain (self._jparameq, gain)


    def set_bypass (self, onoff):
        """
        Set global bypass on (True) or off (False).
        """
        return jackparameq_ext.set_bypass (self._jparameq, onoff)





# ----------------------------------------------------------------------------
#
#  Copyright (C) 2013-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


from zita_jacktools.jackclient import JackClient
from zita_jacktools import jackiecfilt_ext


class JackIECfilt(JackClient):

    """
    IEC Class 1 octave and third octave band filters.
    There can be up to 100 filters, one per output.
    Filters can share inputs.
    """

    OFF  = 0
    OCT1 = 1
    OCT3 = 3

    
    def __init__(self, ninp, nout, client_name, server_name = None):
        """
        Create a new Jackiecfilt instance.

        The result should be checked using get_state().
        The optional 'server_name' allows to select between running
        Jack servers.
        """
        self._jiecfilt, base = jackiecfilt_ext.makecaps (self, client_name, server_name, ninp, nout)
        super().__init__(base)



    def set_filter (self, inp, out, filt, band) :
        """
        Set filter type and band.

        If filt is OFF the output is muted.
        If filt is OCT1, band = 0..9,  freq = 31.5, 63, 125 ... 8k, 16k Hz. 
        if filt is OCT3, band = 0..30, freq = 20, 25, 31.5, 40 ... 16k, 20k Hz.
        """
        return jackiecfilt_ext.set_filter (self._jiecfilt, inp, out, filt, band)





# ----------------------------------------------------------------------------
#
#  Copyright (C) 2008-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


from time import sleep 
from zita_jacktools.jackclient import JackClient
from zita_jacktools import jacksignal_ext


class JackSignal(JackClient):

    """
    Jack client using numpy arrays for input and output.

    The main use of a JackSignal object is to generate
    arbitrary test signals and capture the results.
    There can be up to 100 inputs and outputs.
    In contrast to all other classes in zita_jacktools,
    a JackSignal is created without any inputs or outputs,
    these must be created explicitly while in the PASSIVE
    state.
    """

    TRIGGER = 9

    def __init__(self, client_name, server_name = None) :
        """
        Create a new JackSignal instance.

        The result should be checked using get_state(), if this is
        negative the only option is to delete the object.
        The optional 'server_name' allows to select between running
        Jack servers.
        """
        self._jsignal, base = jacksignal_ext.makecaps (self, client_name, server_name)
        super().__init__(base)
        

    def passive(self):
        """
        Set the state to PASSIVE.

        This is also the initial state after creation, unless it failed.
        In this state the object is a jack client, but the process callback
        does not access any ports. Use this state to create or delete ports.
        Connected output ports in PASSIVE state may output garbage.  
        """
        return jacksignal_ext.set_state (self._jsignal, JackSignal.PASSIVE)


    def silence(self) :
        """
        Set the state to SILENCE.

        In this state the object outputs silence and discards any
        input. Use this state to connect or disconnect ports and
        define signal buffers.
        """
        return jacksignal_ext.set_state (self._jsignal, JackSignal.SILENCE)


    def process(self):
        """
        Set the state to PROCESS.

        In this state the object outputs and records signals until
        the end of the all buffers is reached, taking looping into
        account if specified. At that point the object returns to
        the SILENCE state.
        """
        return jacksignal_ext.set_state (self._jsignal, JackSignal.PROCESS)


    def trigger(self, ind):
        """
        Select the trigger input and set the state to TRIGGER.

        In this state the object is waiting for an external start
        signal. The state will be set to PROCESS by the first sample
        on the trigger input that exceeds +0.5f. Playback and capture
        will start at the position of that sample.
        """
        return jacksignal_ext.set_state (self._jsignal, JackSignal.PROCESS)


    def get_position(self) :
        """
        Return state, frame count.

        The frame count is the number of frames processed
        since entering the process state.
        """
        return jacksignal_ext.get_posit (self._jsignal)


    def wait(self, dt = 0.1):
        """
        Wait while the state is TRIGGER or PROCESS,
        checking every 'dt' seconds.
        """
        actset =  (JackSignal.TRIGGER, JackClient.PROCESS)
        while self.get_state () in actset: sleep (dt); 

            
    def set_output_data(self, ind, data, **kwa):
        """
        Define an output signal.

        Set the playback buffer for the output port with index
        'ind' to 'data'. The 'data' must be a single dimension
        float32 array or array view. To reset, use 'None' as the
        'data' argument. The JackSignal object keeps a reference
        to the array buffer until it is reset or replaced.
        The keyword arguments can be:
        nskip (0): wait 'nskip' samples before starting playback.
        nloop (1): output the 'data' array 'nloop' times.
        """
        try: nloop = kwa ['nloop']
        except: nloop = 1
        try: nskip = kwa ['nskip']
        except: nskip = 0
        return jacksignal_ext.set_output_data (self._jsignal, ind, data, nloop, nskip)


    def set_input_data(self, ind, data, **kwa):
        """
        Define an input signal.

        Set the capture buffer for the input port with index
        'ind' to 'data'. The 'data' must be a single dimension
        float32 array or array view. To reset, use 'None' as the
        'data' argument. The JackSignal object keeps a reference
        to the array buffer until it is reset or replaced.
        The keyword arguments can be:
        nskip (0): wait 'nskip' samples before starting capture.
        nloop (1): write the 'data' array 'nloop' times.
        """
        try: nloop = kwa ['nloop']
        except: nloop = 1
        try: nskip = kwa ['nskip']
        except: nskip = 0
        return jacksignal_ext.set_input_data (self._jsignal, ind, data, nloop, nskip)

    
    def set_output_gain(self, ind, gain):
        """
        Set output gain.
        This can be used to calibrate a physical output.
        The default gain is 1.0.
        """
        return jacksignal_ext.set_output_gain (self._jsignal, ind, gain)

    
    def set_input_gain(self, ind, gain):
        """
        Set input gain.
        This can be used to calibrate a physical input.
        The default gain is 1.0.
        """
        return jacksignal_ext.set_input_gain (self._jsignal, ind, gain)





# ----------------------------------------------------------------------------
#
#  Copyright (C) 2018-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


import globdef as G
from PyQt6 import QtGui, QtCore, QtWidgets


# Base class for the input and output windows.
# These contain a gain control, mute and solo
# buttons, and a channel label.
# This class defines common functionality.
# The GUI layout is defined in the derived classes.


class Gmsbase (QtWidgets.QWidget):

    def __init__(self, parent, callb, index): 
        super (Gmsbase, self).__init__(parent)
        self.callb = callb
        self.index = index
        
    def value_event (self, args):
        self.redraw ()
        self.callb (self.index, G.ACT_GAIN, args.get_value (), 0)     

    def mute_event (self):
        s = self.bmute.get_state () ^ 2
        b = self.bmute.mousebutt
        self.bmute.set_state (s)
        self.callb (self.index, G.ACT_MUTE, s > 0, b)     
        
    def solo_event (self):
        s = self.bsolo.get_state () ^ 1
        b = self.bsolo.mousebutt
        self.bsolo.set_state (s)
        self.callb (self.index, G.ACT_SOLO, s > 0, b)     
        
    def redraw (self):
        gaindb = self.gainctl.get_value ()
        if (gaindb > -200):
            self.gaintxt.setText('%5.1f' % (gaindb,))    
            self.gainctl.set_state (1)
        else:
            self.gaintxt.setText('Off')    
            self.gainctl.set_state (0)    

    def get_state (self):
        m = bmute.get_state () > 0
        return self.gainctl.get_value (), m
    
    def set_state (self, gaindb, mute):
        self.gainctl.set_value (gaindb)
        self.bmute.set_state (2 if mute else 0)
        self.redraw ()

    def get_solo (self):
        return self.bsolo.get_state () > 0
        
    def solo_off (self):
        self.bsolo.set_state (0)
        



# ----------------------------------------------------------------------------
#
#  Copyright (C) 2008-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


from zita_jacktools.jackclient import JackClient
from zita_jacktools import jackmatrix_ext


def db2lin (gain):
    """
    Convert gain in dB to linear value.
    """
    return pow (10.0, 0.05 * gain)


class JackMatrix(JackClient):

    """
    Audio matrix with gain and delay controls for each input,
    output, and input-output pair.

    Gains are linear and can be negative to invert a signal.
    Use db2lin() to set gains in dB. Delays are in seconds.
    Absolute gain values lower than 1e-15 are set to zero. 
    Gain and delay changes are crossfaded over one Jack period.
    """

    def __init__(self, ninp, nout, maxdel, client_name, server_name = None):
        """
        Create a new JackMatrix instance with given number of inputs
        and outputs, and maximum delay. 

        The result should be checked using get_state().
        The maximum number of inputs and outputs is 100.
        The maximum delay is limited to 3 seconds.
        Initial matrix gains are zero.
        Initial input and output gains are one.
        All initial delays are zero.

        The optional 'server_name' allows to select between running
        Jack servers.
        """
        assert (ninp > 0)
        assert (nout > 0)
        self._jmatrix, base = jackmatrix_ext.makecaps (self, client_name, server_name, ninp, nout, maxdel)
        super().__init__(base)


    def set_matrix_gain(self, inp, out, gain) :
        """
        Set gain for matrix element.
        """
        return jackmatrix_ext.set_gain (self._jmatrix, inp, out, gain)


    def set_input_gain(self, inp, gain) :
        """
        Set input gain.
        """
        return jackmatrix_ext.set_gain (self._jmatrix, inp, -1, gain)


    def set_output_gain(self, out, gain) :
        """
        Set output gain.
        """
        return jackmatrix_ext.set_gain (self._jmatrix, -1, out, gain)


    def set_matrix_delay (self, inp, out, delay) :
        """
        Set delay for matrix element.
        """
        return jackmatrix_ext.set_delay (self._jmatrix, inp, out, delay)


    def set_input_delay (self, inp, delay) :
        """
        Set input delay.
        """
        return jackmatrix_ext.set_delay (self._jmatrix, inp, -1, delay)


    def set_output_delay (self, out, delay) :
        """
        Set output delay.
        """
        return jackmatrix_ext.set_delay (self._jmatrix, -1, out, delay)





#!/usr/bin/python

# ----------------------------------------------------------------------------
#
#  Copyright (C) 2008-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


import numpy as np
from math import *
from time import sleep
from zita_jacktools.jacksignal import JackSignal


# ----------------------- Linear sweep  --------------------------
#
# Generate linear sweep and display with jaaa set to peak hold.
# Try using different sweep speeds and jaaa bandwidths.


FMIN = 0
FMAX = 20e3
TIME = 15  # seconds
AMPL = 0.1 # signal amplitude


# Generate a linear sweep signal.
#
def linsweep (fmin, fmax, amp, time, A, fsamp):
    n = int (time * fsamp + 0.1) 
    if n > A.shape [0]: n = A.shape [0]
    a = fmin / fsamp
    b = (fmax - fmin) / (fsamp * n);
    p = 0.0
    for i in range (n):
        A [i] = amp * sin (2 * pi * p)
        a += b
        p += a
        if p > 1.0: p -= 1.0


# Create a JackSignal object and connect.
#
J = JackSignal("JackSignal")
if J.get_state() < 0:
    print ("Failed to create JackSignal -- is Jack running ?")
    exit(1)
J.create_output (0, "out-1")
J.silence()
J.connect_output (0, "jaaa:in_1")

# Get Jack info.
#
name, fsamp, period = J.get_jack_info ()

# Create output buffer.
#
size = int (TIME * fsamp + 0.1)
A = np.empty ([size,], dtype = np.float32)

# Run the test
#
linsweep (FMIN, FMAX, AMPL, TIME, A, fsamp) 
J.set_output_data (0, A)
J.process()
J.wait()
del J




#!/usr/bin/python

# ----------------------------------------------------------------------------
#
#  Copyright (C) 2013-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


import sys
import numpy as np
from time import sleep
from zita_jacktools.jacksignal import JackSignal
sys.path.append ('..')
from utils.sinewave import *


# -----------Generate SMPTE/DIN Intermodulation test signal -----------------

# SMPTE frequencies
#
#Flo = 60
#Fhi = 7000
#Ratio = 4

# DIN frequencies
#
Flo = 250
Fhi = 8000
Ratio = 4

# Level in dB (of lower frequency)
#
Level = -20.0

# Create a Jacksignal object.
#
J = JackSignal("IMtest")
if J.get_state() < 0:
    print ("Failed to create JackSignal -- is the server running ?")
    exit(1)

# Get Jack info.
#
name, Fsamp, period = J.get_jack_info()

# Create one output and connect.
#
J.create_output (0, "out")
J.silence()
J.connect_output (0, "jaaa:in_1")

# Generate test signal. Since we will loop this we need an
# exactly integer number of cycles in the buffer.
#
siglen = int (1.0 * Fsamp + 0.5) # 1 second
Flo = adjust_freq (Flo, Fsamp, siglen)
Fhi = adjust_freq (Fhi, Fsamp, siglen)
# Amplitudes.
Alo = pow (10.0, Level / 20.0)
Ahi = Alo / Ratio
A = (   gen_sinewave (Alo, Flo, Fsamp, siglen)
      + gen_sinewave (Ahi, Fhi, Fsamp, siglen)).astype (np.float32)

# Output signal, loop 1000 times.
#
J.set_output_data (0, A, nloop = 1000)
J.process()
J.wait()

# Cleanup
#    
del A    
del J




#!/usr/bin/python

# ----------------------------------------------------------------------------
#
#  Copyright (C) 2008-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


import numpy as np
from math import *
from time import sleep
from zita_jacktools.jacksignal import JackSignal
from zita_jacktools.jackpeaklim import JackPeaklim
import matplotlib.pyplot as plt


# ------------------------------ Test JackPeaklim ----------------------------
#
# We test the response to
#
#  - a single sample,
#  - a 5 kHz burst,
#  - a 100 Hz burst,
#
# all with a peak amplitude of +6 dB, on top of a
# continuous 1 kHz signal with amplitude -6 dB.
# Zoom in on the plot to see the lookahead at work.
# Note automatically increased release time when
# limiting low frequency signals.


# Create a JackSignal object.
#
J = JackSignal("JackSignal")
if J.get_state() < 0:
    print ("Failed to create JackSignal -- is Jack running ?")
    exit(1)

# Create a JackPeaklim object.
#
L = JackPeaklim (1, "Limiter")
#L.set_release (0.001)    


# Create ports and connect.
#
J.create_output (0, "out-1")
J.create_input (0, "in-1")
J.silence()
J.connect_output (0, "Limiter:in_0")
J.connect_input (0, "Limiter:out_0")

# Get Jack info.
#
name, fsamp, period = J.get_jack_info ()

# Create input and output buffers, 0.3 seconds.
#
L1 = int (0.30 * fsamp + 0.1)
L2 = int (0.04 * fsamp + 0.1)
Aout = np.empty(L1, dtype = np.float32)
Ainp = np.empty(L1, dtype = np.float32)

# Generate output signal.
#

# 1 kHz sine wave at -6 dB during entire buffer.
# Used to see the actual gain. 
#
w = 1e3 * 2 * pi / fsamp
for i in range (L1):
    Aout [i] = 0.5 * sin (w * i)

amp = 2.0 # peak amplitude

# Add singe sample.
Aout [1000] = amp

# Add 5 Khz burst.
w = 5e3 * 2 * pi / fsamp
for i in range (L2):
    Aout [i + 4000] += amp * sin (w * i)

# Add 100 Hz burst.
w = 1e2 * 2 * pi / fsamp
for i in range (L2):
    Aout [i + 9000] += amp * sin (w * i)


# Assign buffers to ports. We skip the first period
# on input to compensate for the loop, and 64 samples
# to compensate for zita-dpl1 latency. The single
# sample peak should end up at sample 1000 in the
# captured signal (assuming Fs = 48 kHz).
#
J.set_output_data (0, Aout)
J.set_input_data (0, Ainp, nskip = period + 64)

# Run the test.
#
J.process()
J.wait()
del J
del L


# Display the result.
#
fig = plt.figure (figsize=(9,6), facecolor='white')
fig.canvas.manager.set_window_title ("peaklim")
ax1 = fig.add_axes ([0.05, 0.05, 0.9, 0.42])
ax1.set_ylim (-1.2, 1.2)
ax1.plot (Ainp, color='b', lw=1)
ax1.grid ()
ax2 = fig.add_axes ([0.05, 0.55, 0.9, 0.42])
ax2.set_ylim (-3.0, 3.0)
ax2.plot (Aout, color='r', lw=1)
ax2.grid ()
plt.show()




#!/usr/bin/python

# ----------------------------------------------------------------------------
#
#  Copyright (C) 2020-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


# Kmeters for Ambisonics signals up to degree 8.

import sys
import signal
from PyQt5 import QtGui, QtCore, QtWidgets
from zita_jacktools.jackkmeter import *
from sys import argv
sys.path.append ('..')
from utils.kmeters import *


class Meterwin(QtWidgets.QWidget):
    
    def __init__(self, nchan, metrics):
        super(Meterwin, self).__init__()
        self.metrics = metrics
        mlist = [ nchan ]
        self.disp = K20meter(self, metrics, mlist, 6, 6)
        self.disp.move(0, 0)
        self.disp.show()
        self.setFixedSize (self.disp.sx, self.disp.sy)
        self.setWindowTitle('Ambisonic levels')
        self.show()        
        self.timer = QtCore.QBasicTimer ()
        self.timer.start (50, self)

    def timerEvent (self, ev):
        st, rms, dpk = K20.get_levels ()
        self.disp.rms = rms
        self.disp.pks = dpk
        self.disp.update ()

if len (argv) < 2:
    print ("mcmeter <channels>")
    exit (1)

chan = int (argv [1])
chan = min (64, max (1, chan))

        
# Create K-meter processors. 
K20 = JackKmeter (chan, 'Meters')

# Allow Ctrl-C        
signal.signal (signal.SIGINT, signal.SIG_DFL)

# Set dark window background.
app = QtWidgets.QApplication(sys.argv)
pal = app.palette()
bgc = QtGui.QColor (40, 40, 40)
pal.setColor(pal.Window, bgc)
app.setPalette(pal)

# Create meter window.
metrics = makek20metrics (bgc)
win = Meterwin (chan, metrics)
sys.exit(app.exec_())





# ----------------------------------------------------------------------------
#
#  Copyright (C) 2008-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


from zita_jacktools.jackclient import JackClient
from zita_jacktools import jackplayer_ext


class JackPlayer(JackClient):

    """"
    Multichannel audio file player, with high quality resampling.
    Supports up to 100 channels. 
    """
    
    STOPPED = 10
    STOPLOC = 11
    PLAYING = 12
    PLAYLOC = 13
    ENDFILE = 14

    def __init__(self, nchan, client_name, server_name = None) :
        """
        Create a new JackPlayer instance with 'nchan' outputs.

        The result should be checked using get_state().
        The optional 'server_name' allows to select between running
         servers.
        """
        self._jplayer, base = jackplayer_ext.makecaps (self, client_name, server_name, nchan)
        super().__init__(base)


    def get_position (self) :
        """
        Return state, file position.

        Additional states are:

        STOPPED = 10    Player is active but stopped.
        STOPLOC = 11    Waiting for locate() to finish.    
        PLAYING = 12    Player is playing the audiofile.
        PLAYLOC = 13    Waiting for locate(), play when ready.
        ENDFILE = 14    Player stopped at end of file.

        The file position is in frames at the audio file sample rate.
        """
        return jackplayer_ext.get_posit (self._jplayer)


    def silence(self) :
        """
        Goto the SILENCE state.

        In this state the player outputs silence and does not respond to any
        transport commands. Use this state to connect or disconnect ports,
        and to open or close audio files.
        """
        return jackplayer_ext.set_state (self._jplayer, JackClient.SILENCE)


    def stop(self) :
        """
        Stop playback with transport commands enabled.
        """
        return jackplayer_ext.set_state (self._jplayer, JackPlayer.STOPPED)


    def play(self) :
        """
        Start playback.
        """
        return jackplayer_ext.set_state (self._jplayer, JackPlayer.PLAYING)


    def set_gain(self, gain, time = 0) :
        """
        Set playback gain in dB.

        Gain will change smoothly to the given value in 'time'
        seconds. Values less than -150 dB will result in a complete
        mute. The initial gain is 0 dB.
        """
        return jackplayer_ext.set_gain (self._jplayer, gain, time)


    def locate(self, posit) :
        """
        Locate to position in frames at the audio file sample rate.

        This can be used while stopped or while playing. If used in
        PLAYING state, playback resumes when at least one second or
        all remaining audio has been buffered.
        """
        return jackplayer_ext.set_posit (self._jplayer, posit)


    def open_file(self, name) :  
        """
        Open an audio file. Player must be in SILENCE state.
        Returns zero on success, non-zero otherwise.
        """
        return jackplayer_ext.open_file (self._jplayer, name)


    def close_file(self) :  
        """
        Close the current audio file. Player must be in SILENCE state.
        Returns zero on success, non-zero otherwise.
        """
        return jackplayer_ext.close_file (self._jplayer)


    def get_file_info(self) :
        """
        Returns channels, sample rate, lenght in frames.
        """
        return jackplayer_ext.get_file_info (self._jplayer)





# ----------------------------------------------------------------------------
#
#  Copyright (C) 2021-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


from zita_jacktools.jackclient import JackClient
from zita_jacktools import jackgainctl_ext


class JackGainctl(JackClient):

    """
    Multichannel dezippered audio gain control, with per
    channel and master gain and mute controls.

    The 'gain' and 'mute' settings are independent. A gain
    less than -120 dB is equivalent to 'off' (-inf dB).
    The initial master gain is 'off', not muted.
    The initial channel gain is 0 dB, not muted.

    Gain changes are dezippered as follows:
    'off' -> 'on'  : 10 ms fade in.
    'on'  -> 'off' : 10 ms fade out.
    otherwise      : gain changes by max 6 dB in each 10 ms.
    """

    def __init__(self, nchan, client_name, server_name = None):
        """
        Create a new JackGainctl instance.

        The result should be checked using get_state().
        The optional 'server_name' allows to select between running
        Jack servers.
        """
        assert (nchan > 0)
        self._jgainctl, base = jackgainctl_ext.makecaps (self, client_name, server_name, nchan)
        super().__init__(base)


    def set_gain(self, gain) :
        """
        Set master gain in dB.
        """
        return jackgainctl_ext.set_gain (self._jgainctl, -1, gain)


    def set_mute(self, mute) :
        """
        Set master mute state (boolean). 
        """
        return jackgainctl_ext.set_mute (self._jgainctl, -1, mute)


    def set_channel_gain(self, chan, gain) :
        """
        Set channel gain in dB.
        """
        assert (chan >= 0)
        return jackgainctl_ext.set_gain (self._jgainctl, chan, gain)


    def set_channel_mute(self, chan, mute) :
        """
        Set channel mute state (boolean). 
        """
        assert (chan >= 0)
        return jackgainctl_ext.set_mute (self._jgainctl, chan, mute)





#!/usr/bin/python

# ----------------------------------------------------------------------------
#
#  Copyright (C) 2013-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


import numpy as np
import matplotlib.pyplot as plt
from math import *
from time import sleep
from zita_jacktools.jackparameq import JackParameq
from zita_jacktools.jacksignal import JackSignal


# Measure frequency response of parametric filter in JackParameq.
#

# Create and connect objects.
#
F = JackParameq (1, "123", "JackParameq")
if F.get_state() < 0:
    print ("Failed to create JackSignal -- is Jack running ?")
    exit(1)

J = JackSignal("JackSignal")
J.create_output (0, "out-1")
J.create_input (0, "in-1")
J.silence()
J.connect_output (0, "JackParameq:in_0")
J.connect_input (0, "JackParameq:out_0")

# Get Jack info.
#
name, fsamp, period = J.get_jack_info ()

# Parameters.
#
impval = 1.0
fftlen = 4 * 1024

# Generate data.
#
Aout = np.zeros ([100,], dtype = np.float32)
Aout [0] = impval

Ain1 = np.empty ([fftlen,], dtype = np.float32)
Freq = np.linspace (0, fsamp / 2, num = fftlen // 2 + 1)

J.set_output_data (0, Aout)
J.set_input_data (0, Ain1, nskip = period)

# Set up plot.
#
fig = plt.figure (figsize=(10,6), facecolor='white')
fig.canvas.manager.set_window_title ('parameq')
xx = [0.04, 0.54, 0.54]
yy = [0.54, 0.54, 0.05]

# Measurements.
#
F.set_bypass (False)
for i in range (3):
    ax = fig.add_axes ([xx[i], yy[i], 0.45, 0.44])
    ax.set_xlim (20, 24e3)
    ax.set_ylim (-25, 25)
    ax.set_xscale ('log')
    ax.text (30, 22, "Type %d" % (i + 1,))
    ax.grid ()

    print ("Type %d" % (i + 1,))
    # Loop over gain range.
    for g in range (-20,21,5):
        # Set section 'i' parameters.
        F.set_filter (i, 1e3, g, 1.0)
        # Wait for smooth change to end.
        if g == -20: sleep (0.5)
        else:        sleep (0.2)
        print ("  g = %5.1f" % (g,))
        # Measure.
        J.process()
        J.wait()
        # Plot result.
        Spec = np.fft.rfft (Ain1)
        ax.plot (Freq, 20 * np.log10 (np.abs (Spec) + 1e-10), color='b', lw=1)
    # Set section to bypass.    
    F.set_filter (i, 1e3, 0, 0.5)

del F, J
        
fig.text (0.02, 0.40, "Type 1: Constant bandwidth resonance added or subtracted.")    
fig.text (0.02, 0.36, "Type 2: As mode 0, but adjusted for symmetry if gain is negative.")    
fig.text (0.02, 0.32, "Type 3: Bandwidth depends on gain, same as mode 1 at +/- 10 dB.")    
# Show result.
#    
plt.show()



# ----------------------------------------------------------------------------
#
#  Copyright (C) 2013-2020 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


import numpy as np
from math import pi, floor, atan2
from cmath import polar


def adjust_freq (freq, fsamp, nsamp):
    """ 
    Adjust 'freq' for an integer number of cycles in 'nsamp' samples.
    """
    t = nsamp / fsamp
    k = int (freq * t + 0.5)
    return k / t


def gen_sinewave (ampl, freq, fsamp, nsamp):
    """
    Generate sine wave test signal.
    """
    # Phase array, better use double precision for this.
    w = 2 * pi * freq / fsamp
    R = np.linspace (0, w * nsamp, nsamp, endpoint = False, dtype = np.float64)
    return ampl * np.sin (R)
        

def gen_complex (freq, fsamp, nsamp, window = True):
    """
    Generate windowed reference signal for selective measurement.
    """
    # Phase array, better use double precision for this.
    w = 2 * pi * freq / fsamp
    R = np.linspace (0, w * nsamp, nsamp, endpoint = False, dtype = np.float64)
    # Generate complex (cos(R), sin(R))
    S = np.exp ((0 + 1j) * R)
    if window:
        # Raised cosine window.
        W = np.linspace (0, 2 * pi, nsamp, endpoint = False, dtype = np.float32)
        S *= (1 - np.cos (W))
    return S


def sigdetect (Ainp, Aref):
    """
    Frequency selective measurement. Detect signal in Ainp at the
    frequency determined by Aref, generated by 'gen_complex' above.
    Equivalent to one bin of an FFT.
    Returns amplitude and phase in radians.
    """
    return polar (2 * np.dot (Ainp, Aref) / Aref.shape [0])





# ----------------------------------------------------------------------------
#
#  Copyright (C) 2008-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


from zita_jacktools import jackclient_ext


class JackClient() :

    """
    Base class for most jacktools classes.
    Do not instantiate directly.
    """

    INITIAL =  0
    PASSIVE =  1
    SILENCE =  2
    PROCESS = 10
    FAILED  = -1
    ZOMBIE  = -2

    
    def __init__(self, jclient) :
        """
        For use by derived classes only !!
        Store a pointer to the C++ base class.
        """
        self._jclient = jclient


    def get_state(self) :
        """
        Returns the current state. Possible states are:
        
        PASSIVE =  1  Running as Jack client, ports are not accessed.
        SILENCE =  2  Outputs are silent, inputs are ignored. 
        PROCESS = 10  Normal processing state. 
        FAILED  = -1  Failed to intialise or run-time error.
        ZOMBIE  = -2  Kicked out by Jack.

        Most jacktools classes start up in the PROCESS state, and don't 
        have the PASSIVE or SILENCE states. Some clients have additional
        states.

        The only option in the last two states is to delete the object.
        """
        return jackclient_ext.get_state (self._jclient)

    def get_jack_info(self) :
        """
        Return the jack client name, sample rate, period size.
        """
        return jackclient_ext.get_jack_info (self._jclient)

    def create_input(self, ind, name) :
        """
        Create an input port with index 'ind' and name 'name'.
        Indices start at zero. Possible in the PASSIVE state only.
        """
        return jackclient_ext.port_operation (self._jclient, 0, ind, name)

    def create_output(self, ind, name) :
        """
        Create an output port with index 'ind' and name 'name'.
        Indices start at zero. Possible in the PASSIVE state only.
        """
        return jackclient_ext.port_operation (self._jclient, 1, ind, name)
    
    def delete_input(self, ind) :
        """
        Delete the input port with index 'ind', or all input
        ports if 'ind' == -1. Possible in the PASSIVE state only.
        """
        return jackclient_ext.port_operation (self._jclient, 2, ind, 0)

    def delete_output(self, ind) :
        """
        Delete the input port with index 'ind', or all input
        ports if 'ind' == -1. Possible in the PASSIVE state only.
        """
        return jackclient_ext.port_operation (self._jclient, 3, ind, 0)

    def rename_input(self, ind, name) :
        """
        Rename input port with index 'ind' to 'name'.
        Indices start at zero.
        """
        return jackclient_ext.port_operation (self._jclient, 4, ind, name)

    def rename_output(self, ind, name) :
        """
        Rename output port with index 'ind' to 'name'.
        Indices start at zero.
        """
        return jackclient_ext.port_operation (self._jclient, 5, ind, name)

    def connect_input(self, ind, srce) :
        """
        Connect the input with index 'ind' to 'srce'.
        """
        return jackclient_ext.port_operation (self._jclient, 6, ind, srce)

    def connect_output(self, ind, dest) :
        """
        Connect the output with index 'ind' to 'dest'.
        """
        return jackclient_ext.port_operation (self._jclient, 7, ind, dest)

    def disconn_input (self, ind, srce = None) :
        """
        Disconnect an input port, or all input ports if 'ind' == -1.
        In the first case, if 'srce' is not None, only the named
        source is disconnected.
        """
        return jackclient_ext.port_operation (self._jclient, 8, ind, srce)

    def disconn_output (self, ind, dest = None) :
        """
        Disconnect an output port, or all output ports if 'ind' == -1.
        In the first case, if 'dest' is not None, only the named
        destination is disconnected.
        """
        return jackclient_ext.port_operation (self._jclient, 9, ind, dest)





# ----------------------------------------------------------------------------
#
#  Copyright (C) 2010-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


from zita_jacktools.jackclient import JackClient
from zita_jacktools import jackambpan_ext


class JackAmbpan(JackClient):

    """
    Ambisonic panners and mixing bus for up to 100 inputs.

    ACN, SN3D, up to degree 8.
    """

    def __init__(self, degree, ninput, client_name, server_name = None):
        """
        Create a new JackAmbpan instance.

        The result should be checked using get_state().
        The optional 'server_name' allows to select between running
        Jack servers.
        """
        assert (degree >= 1)
        assert (degree <= 8)
        self._jambpan, base = jackambpan_ext.makecaps (self, client_name, server_name, degree, ninput)
        super().__init__(base)


    def set_direction(self, inp, azim, elev, time = 0.1) :
        """
        Set panning direction for given channel.

        Angles are in degrees.
        Panning gains will be interpolated linearly over 'time' seconds.
        """
        return jackambpan_ext.set_direction (self._jambpan, inp, azim, elev, time)




#!/usr/bin/python

# ----------------------------------------------------------------------------
#
#  Copyright (C) 2008-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


import numpy as np
from time import sleep
from zita_jacktools.jackplayer import JackPlayer


# Same as playfile1.py, but using JackPlayer.
# This will resample to the Jack sample rate if necessary.
# Skip 10 seconds then play 20 seconds of a stereo audio file.


P = JackPlayer (2, "Player")
if P.get_state () < 0:
    print ("Failed to create JackPlayer -- is the server running ?")
    exit (1)

P.connect_output (0, "system:playback_1")
P.connect_output (1, "system:playback_2")

# Replace by your favourite track...
#
if P.open_file ("/audio/audiofiles/tracks/susheela_raman-maya.wav"):
    print ("Failed to open audio file.")
    exit (1)

name, fsjack, period = P.get_jack_info ()
chan, fsfile, lenght = P.get_file_info ()

if fsjack != fsfile:
    print ("Resampling %d -> %d." % (fsfile, fsjack))

P.stop ()
P.set_gain (-20) # dB
P.locate (10 * fsfile)
P.play ()
while True:
    state, posit = P.get_position ()
    if state < JackPlayer.PLAYING or state == JackPlayer.ENDFILE or posit >= 30 * fsfile: break
    sleep (0.1)
P.stop ()
P.silence ()
P.close_file ()
del P




# ----------------------------------------------------------------------------
#
#  Copyright (C) 2018-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


from PyQt5 import QtWidgets, QtGui, QtCore


# Style class for buttons.
#
class ButtonStyle ():
    
    def __init__(self, sx, sy, bgcol, fgcol, mkcol = None, font = None, pixs = None):
        self.pixmap = ButtonStyle.make_pixmap (sx, sy, bgcol, fgcol)
        self.mkcol = mkcol
        self.nstate = len (fgcol)
        self.dx = self.pixmap.width () // self.nstate
        self.dy = self.pixmap.height ()
        self.font = font
        self.pixs = pixs
        
        
    @classmethod    
    def make_pixmap (cls, sx, sy, bgcol, fgcol):
        nc = len (fgcol)
        sx += 1
        sy += 1
        pm = QtGui.QPixmap (nc * sx, sy);
        pm.fill (bgcol)
        qp = QtGui.QPainter (pm)
        qp.setRenderHint (QtGui.QPainter.Antialiasing)
        sh = QtGui.QColor (0, 0, 0, 180)

        for i in range (nc):
            qp.fillRect (2, 2, sx - 2, sy - 2, sh)
            cg = QtGui.QLinearGradient (0, sy / 2, 0, 0)
            cg.setColorAt (0.0, fgcol [i].darker (110))
            cg.setColorAt (0.6, fgcol [i])
            cg.setColorAt (1.0, fgcol [i].lighter (115))
            qp.fillRect (1, 1, sx - 2, sy - 2, cg)
            qp.translate (sx, 0)
        qp.end ()       
        return pm


# Base class for buttons.
#
class ButtonBase (QtWidgets.QWidget):

    bpressEvent = QtCore.pyqtSignal(object)
    brelseEvent = QtCore.pyqtSignal(object)
    modifiers = None
    mousebutt = None
    
    def __init__(self, parent, style, inpen = False, index = None):
        super (ButtonBase, self).__init__(parent)
        self.resize (style.dx - 1, style.dy - 1)
        self.style = style
        self.inpen = inpen
        self.state = 0
        self.index = index
        self.text = None

    def set_text (self, t):
        if t is None: self.text = t;
        else:
            self.text = t.split ('\n')
            st = self.style
            ft = st.font
            ft.setPixelSize (st.pixs)
            fm = QtGui.QFontMetrics (ft)
            a = fm.ascent ()
            h = fm.descent () + a
            n = len (self.text)
            d = (st.dy - 2 - n * h) / (n + 1)
            self.xt = [(st.dx - fm.width (t) - 2) / 2 for t in self.text] 
            self.yt = a + d 
            self.dy = h + d
        self.update ()
            
    def set_state (self, s):
        s = (s & 255) | (self.state & 256)
        if s != self.state:
            self.state = s
            self.update ()
            
    def get_state (self):
        return self.state & 255
            
    def mousePressEvent (self, E):
        ButtonBase.modifiers = E.modifiers ()
        ButtonBase.mousebutt = E.button()
        if self.inpen:
            self.state |= 256
            self.update ()
            self.bpressEvent.emit (self)
        
    def mouseReleaseEvent (self, E):
        ButtonBase.modifiers = E.modifiers ()
        ButtonBase.mousebutt = E.button()
        if self.inpen:
            self.state &= 255
            self.update ()
            self.brelseEvent.emit (self)
        
    def paintEvent (self, e):
        st = self.style
        pm = st.pixmap
        ks = self.state & 255
        dd = self.state >> 8
        qp = QtGui.QPainter (self)
        qp.drawPixmap (0, 0, pm, ks * st.dx + 1 - dd, 1 - dd, st.dx - 1, st.dy - 1)
        if self.text is not None:
            ft = st.font
            ft.setPixelSize (st.pixs)
            qp.setFont (ft)
            qp.setPen (st.mkcol [ks])
            n = len (self.text)
            y = self.yt + dd
            for i in range (n):
                qp.drawText (QtCore.QPointF (self.xt [i] + dd, y), self.text [i])
                y += self.dy
        qp.end ()




#!/usr/bin/python

# ----------------------------------------------------------------------------
#
#  Copyright (C) 2018-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


import sys
import signal
import globdef
from PyQt6 import QtWidgets
from zita_jacktools.jackmatrix import JackMatrix
from mainwin import Mainwindow


# Define input and output port names.
#
inputs = [ '1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', '12' ]
outputs = [ 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H' ]
jackname = 'Matrix'


# Create the JackMatrix.
#
jmatrix = JackMatrix (len (inputs), len (outputs), 0.1, jackname)


# Connect inputs and outputs.
#
#jmat.connect_input (0, '...')
#jmat.connect_input (1, '...')
#
#jmat.connect_output (0, '...')
#jmat.connect_output (1, '...')


# Start PyQt5 app.
#
signal.signal (signal.SIGINT, signal.SIG_DFL)
app = QtWidgets.QApplication (sys.argv)
# Comment next line for default light theme.
globdef.darkpal (app)
Mainwindow (inputs, outputs, jmatrix, jackname)
sys.exit (app.exec())





#!/usr/bin/python

# ----------------------------------------------------------------------------
#
#  Copyright (C) 2008-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


import numpy as np
from zita_audiotools.audiofile import AudioFile
from zita_jacktools.jacksignal import JackSignal


# Simple example using Audiofile and Jacksignal.
# Skip 10 seconds then play 20 seconds of a stereo audio file.


F = AudioFile()
# Replace by your favourite track...
#
rv = F.open_read("/audio/audiofiles/tracks/susheela_raman-maya.wav")
if rv == None:
    print ("Failed to open audio file.")
    exit (1)

F.seek (10 * F.sampfreq ())
nsam = 20 * F.sampfreq ()
A2 = np.empty ([nsam, 2], dtype = np.float32)
F.read (A2)
F.close ()

J = JackSignal ("JackSignal")
if J.get_state () < 0:
    print ("Failed to create JackSignal -- is the server running ?")
    exit (1)

J.create_output (0, "out-1")
J.create_output (1, "out-2")
J.silence()
J.connect_output (0, "system:playback_1")
J.connect_output (1, "system:playback_2")

# Assign slices to outputs.
#
J.set_output_data (0, A2 [:,0])  
J.set_output_data (1, A2 [:,1])

# Set gains.
#
J.set_output_gain (0, 0.1)
J.set_output_gain (1, 0.1)

# Play.
#
J.process ()
J.wait ()



#!/usr/bin/python

# ----------------------------------------------------------------------------
#
#  Copyright (C) 2013-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


import sys
import matplotlib.pyplot as plt
import numpy as np
from time import sleep
from math import hypot, log10
from zita_jacktools.jacksignal import JackSignal
sys.path.append ('..')
from utils.sinewave import *


# ---------------- SMPTE/DIN Intermodulation measurement --------------------
#
# Run jack_nonlin with arguments 0.01 0.01 0


# SMPTE frequencies
#
#Flo = 60
#Fhi = 7000

# DIN frequencies
#
Flo = 250
Fhi = 8000


# Test levels (dB, level of low frequency) 
#
Levels = [ -50, -45, -40, -35, -30, -25, -20, -15, -10, -5, 0 ]
Mtime = 1.0


# Create a Jacksignal object.
#
J = JackSignal("IMtest")
if J.get_state() < 0:
    print ("Failed to create JackSignal -- is the server running ?")
    exit(1)

# Get Jack info.
#
name, Fsamp, period = J.get_jack_info()

# Create ports and connect
#
J.create_output (0, "out")
J.create_input (0, "in")
J.silence()
J.connect_output (0, "jack_nonlin:in")
J.connect_input (0, "jack_nonlin:out")

# Length of test signals in samples.
#
siglen = int (Mtime * Fsamp + 0.5)
margin = 1000
buflen = siglen + margin

# Generate test signal, will be scaled for output.
#
Aout = (  gen_sinewave (1.00, Flo, Fsamp, buflen)
        + gen_sinewave (0.25, Fhi, Fsamp, buflen)).astype (np.float32)

# Input buffer.
#
Ainp = np.zeros ((buflen,), dtype = np.float32)

# Generate reference signals for selective measurement.
#
Mlo = gen_complex (Flo, Fsamp, siglen)
Mhi = gen_complex (Fhi, Fsamp, siglen)
M3a = gen_complex (Fhi - 2 * Flo, Fsamp, siglen)
M2a = gen_complex (Fhi - 1 * Flo, Fsamp, siglen)
M2b = gen_complex (Fhi + 1 * Flo, Fsamp, siglen)
M3b = gen_complex (Fhi + 2 * Flo, Fsamp, siglen)


# Run test.
#
N = len (Levels)
D2 = [ 0 for i in range (N) ]
D3 = [ 0 for i in range (N) ]

for i in range (N):

    # Define signal buffers and run test. 
    amp = pow (10.0, Levels [i] / 20.0) 
    J.set_output_data (0, amp * Aout)
    J.set_input_data (0, Ainp) 
    J.process()
    J.wait()

    # Skip margin samples and measure.    
    # We are not interested in the phase here.
    #
    T = Ainp [margin:]
    Llo,p = sigdetect (T, Mlo)
    Lhi,p = sigdetect (T, Mhi)
    L3a,p = sigdetect (T, M3a)
    L2a,p = sigdetect (T, M2a)
    L2b,p = sigdetect (T, M2b)
    L3b,p = sigdetect (T, M3b)

    # Print results.
    #
    dblo = 20 * log10 (Llo + 1e-30)
    dbhi = 20 * log10 (Lhi + 1e-30)
    D2 [i] = im2 = 100 * hypot (L2a, L2b) / Lhi
    D3 [i] = im3 = 100 * hypot (L3b, L3b) / Lhi
    print ("Level = %5.1f    Al = %5.1f  Ah = %5.1f   im2 = %6.3f%%  im3 = %6.3f%%" % (Levels [i], dblo, dbhi, im2, im3))

# Stop the Jacksignal process.
del J
    
# Create a nice graph to present the results.
#
Levels = np.array (Levels)
fig = plt.figure (figsize=(8,5), facecolor='white')
ax = fig.add_axes ([0.07, 0.06, 0.86, 0.88])
ax.set_title ("% Intermodulation distortion (DIN)")
ax.set_xlim (-55, 5)
ax.set_ylim (1e-3, 1e1)
ax.set_yscale ('log')
ax.bar (Levels - 0.9, D2, 0.8, color = 'b')
ax.bar (Levels + 0.1, D3, 0.8, color = 'g')
ax.text (-51, 5.0, 'IM2', size = 17, color = 'b')
ax.text (-46, 5.0, 'IM3', size = 17, color = 'g')
ax.grid ()
plt.show ()
    



# ----------------------------------------------------------------------------
#
#  Copyright (C) 2018-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


from PyQt6 import QtWidgets, QtGui, QtCore


# Style class for rotary controls.
#
class RotaryStyle ():
    
    def __init__(self, rad, bgcol, fgcol, mkcol):
        self.pixmap = RotaryStyle.make_pixmap (rad, bgcol, fgcol, mkcol)
        self.mkcol = mkcol
        self.nstate = len (fgcol)
        self.sx = self.pixmap.width () // self.nstate
        self.sy = self.pixmap.height ()

    @classmethod    
    def make_pixmap (cls, rad, bgcol, fgcol, mkcol):
        nc = len (fgcol)
        r = int (rad)
        s = 2 * r + 5
        pm = QtGui.QPixmap (nc * s, s);
        pm.fill (bgcol)
        qp = QtGui.QPainter (pm)
        qp.translate (s / 2, s / 2)
        qp.setRenderHint (qp.RenderHint.Antialiasing)
        sh = QtGui.QColor (0, 0, 0, 180)
        for i in range (nc):
            qp.setPen (QtCore.Qt.PenStyle.NoPen)
            qp.setBrush (sh)
            qp.drawEllipse (2 - rad, 2 - rad, 2 * rad, 2 * rad)
            cg = QtGui.QRadialGradient (0, 0, rad, -5, -5, 1)
            cg.setColorAt (1.0, fgcol [i].darker (110))
            cg.setColorAt (0.5, fgcol [i])
            cg.setColorAt (0.0, fgcol [i].lighter (110))
            qp.setBrush (cg)
            qp.drawEllipse (-rad, -rad, 2 * rad, 2 * rad)
            qp.translate (s, 0)
        qp.end ()       
        return pm
        
        
# Base class for rotary controls.
#
class RotaryBase (QtWidgets.QWidget):

    valueEvent = QtCore.pyqtSignal(object)
    pressEvent = QtCore.pyqtSignal(object)
    relseEvent = QtCore.pyqtSignal(object)
    modifiers = None
    mousebutt = None
    xref = 0
    yref = 0
    kref = 0
    
    def __init__(self, parent, style):
        super (RotaryBase, self).__init__(parent)
        self.resize (style.sx, style.sy)
        self.style = style
        self.s = 0
        self.w = 0
        self.k = 0
        self.a = self.k2a (self.k)
        self.v = self.k2v (self.k)

    def get_state (self):
        return self.s
            
    def set_state (self, s):
        self.s = s
        self.update ()
            
    def get_value (self):
        return self.v

    def set_value (self, v):
        self.k = self.v2k (v)
        self.a = self.k2a (self.k)
        self.v = self.k2v (self.k)
        self.update ()
    
    def mousePressEvent (self, E):
        RotaryBase.modifiers = E.modifiers()
        RotaryBase.mousebutt = E.button()
        RotaryBase.kref = self.k
        P = E.position ()
        RotaryBase.xref = P.x()
        RotaryBase.yref = P.y()
        self.pressEvent.emit (self)
    
    def mouseReleaseEvent (self, E):
        RotaryBase.modifiers = E.modifiers()
        RotaryBase.mousebutt = E.button()
        self.relseEvent.emit (self)

    def mouseMoveEvent (self, E):
        P = E.position ()
        d = (P.x() - RotaryBase.xref) - (P.y() - RotaryBase.yref)
        self.handle_move (d)
        self.a = self.k2a (self.k)
        self.v = self.k2v (self.k)
        self.update ()
        self.valueEvent.emit (self)
    
    def wheelEvent (self, E):
        RotaryBase.modifiers = E.modifiers ()
        w = self.w + E.angleDelta().y ()
        if abs (w) >= 60:
            self.handle_wheel (w)
            w = 0
        self.w = w    
        self.a = self.k2a (self.k)
        self.v = self.k2v (self.k)
        self.update ()
        self.valueEvent.emit (self)
        
    def paintEvent (self, E):
        st = self.style
        pm = st.pixmap
        s = self.s
        qp = QtGui.QPainter ()
        qp.begin (self)
        qp.setRenderHint (QtGui.QPainter.RenderHint.Antialiasing)
        qp.drawPixmap (0, 0, pm, s * st.sx, 0, st.sx, st.sy)
        qp.setPen (QtGui.QPen (st.mkcol [s], 1.3))
        qp.translate (st.sx / 2, st.sy / 2)
        qp.rotate (-self.a)
        qp.drawLine (0, 0, st.sx // 2 - 3, 0)
        qp.end ()

        



#!/usr/bin/python

# ----------------------------------------------------------------------------
#
#  Copyright (C) 2012-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


import numpy as np
import matplotlib.pyplot as plt
from zita_jacktools.jackiecfilt import JackIECfilt
from zita_jacktools.jacksignal import JackSignal


# Measure frequency response of third octave band filters in JackIECfilt.
#

# We measure four filters at a time.
# Create and connect objects.
#
J = JackSignal("JackSignal")
if J.get_state() < 0:
    print ("Failed to create JackSignal -- is Jack running ?")
    exit(1)
J.create_output (0, "out")
J.create_input (0, "in")
J.silence()

F = JackIECfilt (1, 4, "JackIECfilt")
F.connect_input (0, "JackSignal:out")
for i in range (4):
    F.connect_output (i, "JackSignal:in")

# Get Jack info.
#
name, fsamp, period = J.get_jack_info ()

# Parameters
#
fftlen = 64 * 1024

# Generate data.
#
Aout = np.zeros ([10,], dtype = np.float32)
Aout [0] = 1.0
Ain1 = np.empty ([fftlen,], dtype = np.float32)
Freq = np.linspace (0, fsamp / 2, num = fftlen // 2 + 1)

J.set_output_data (0, Aout)
J.set_input_data (0, Ain1, nskip = period)

# Set up plot.
#
fig = plt.figure (figsize=(8,5), facecolor='white')
fig.canvas.manager.set_window_title ('oct3band')
ax1 = fig.add_axes ([0.06, 0.06, 0.90, 0.90])
ax1.set_xlim (1e1, 24e3)
ax1.set_ylim (-50, 5)
ax1.set_xscale ('log')
ax1.grid ()


# Measure and plot all bands, four at a time.
#
for i in range (0, 8):
    print ("measuring bands", end = "")
    for j in range (0, 4):
        k = i + 8 * j
        if k <= 30:
            F.set_filter (0, j, 3, k)
            print (" %2d" % (k,), end = "") 
        else:
            F.set_filter (0, j, 0, 0)
    print ()
    J.process()
    J.wait()
    Spec = np.fft.rfft (Ain1)
    ax1.plot (Freq, 20 * np.log10 (np.abs (Spec) + 1e-10), color='b', lw=1)

# Show result.
#    
del J, F
plt.show()



# ----------------------------------------------------------------------------
#
#  Copyright (C) 2018-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


from PyQt6 import QtGui, QtCore, QtWidgets
from rotarybase import *


# Linear control, v0...v1 in ns steps.
#
class Rlinctl (RotaryBase):

    def __init__(self, parent, style, v0, v1, ns, dk):
        self.v0 = v0
        self.dv = v1 - v0
        self.ns = ns
        self.dk = dk
        super (Rlinctl, self).__init__(parent, style)
        
    def handle_wheel (self, dw):
        sh = self.modifiers ==  QtCore.Qt.KeyboardModifier.ShiftModifier
        if sh: dk = 1
        else:  dk = self.dk
        if dw < 0: dk = -dk
        self.k = max (0, min (self.ns, self.k + dk))
        
    def handle_move (self, dk):
        if self.mousebutt == QtCore.Qt.MouseButton.LeftButton:
            self.k = max (0, min (self.ns, self.kref + dk // 2))
        
    def k2a (self, k):
        return 225 - 270 * k / self.ns

    def k2v (self, k):
        return self.v0 + self.dv * k / self.ns
        
    def v2k (self, v):
        r = (v - self.v0) / self.dv
        return max (0, min (self.ns, int (self.ns * r)))


# Audio gain control
#
class Rgainctl (RotaryBase):

    def __init__(self, parent, style):
        super (Rgainctl, self).__init__(parent, style)
        
    def handle_wheel (self, dw):
        sh = self.modifiers == QtCore.Qt.KeyboardModifier.ShiftModifier 
        if sh: dk = 1   # 0.5 dB
        else:  dk = 4   # 2.0 dB
        if dw < 0: dk = -dk
        self.k = max (0, min (120, self.k + dk))
        
    def handle_move (self, dk):
        if self.mousebutt == QtCore.Qt.MouseButton.LeftButton:
            self.k = max (0, min (120, self.kref + dk // 2))
        
    def k2a (self, k):
        return 225 - 270 * k / 120

    def k2v (self, k):
        if k > 0: return k / 2 - 50.0
        else:     return -200.0
        
    def v2k (self, v):
        return max (0, min (160, int (2 * v + 100.5)))




#!/usr/bin/python

# ----------------------------------------------------------------------------
#
#  Copyright (C) 2012-2018 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


import numpy as np
from zita_jacktools.jacknmeter import JackNmeter
from zita_jacktools.jacksignal import JackSignal
import matplotlib.pyplot as plt


# Test JackNmeter filters
# -----------------------


# Create a JackSignal object.
#
S = JackSignal ("JackSignal")
if S.get_state () < 0:
    print ("Failed to create JackSignal -- is Jack running ?")
    exit(1)

# Create a JackNmeter object.
#
M = JackNmeter (1, 1, "JackNmeter")
if M.get_state () < 0:
    print ("Failed to create JackNmeter -- is Jack running ?")
    exit(1)

# Create and connect ports.
#
S.create_output (0, "out-1")
S.create_input (0, "in-1")
S.silence()
S.connect_output (0, "JackNmeter:in_0")
S.connect_input (0, "JackNmeter:out_0")

# Get Jack info.
#
name, fsamp, period = S.get_jack_info ()

# Parameters
#
impval = 1.0
fftlen = 16384

# Create and assign buffers.
#
Ain1 = np.empty ([fftlen,], dtype = np.float32)
Aout = np.zeros ([100,], dtype = np.float32)
Aout [0] = impval
S.set_input_data (0, Ain1, nskip = period - 10) # Skip one period minus 10 samples.
S.set_output_data (0, Aout)
Freq = np.linspace (0, fsamp / 2, num = fftlen // 2 + 1)

# Run one test.
#
def run_test (ftype, dcfilt, title):
    M.set_input (0, 0);
    M.set_filter (0, ftype, dcfilt);
    S.process()
    S.wait()
    Spec = np.fft.rfft (Ain1)
    fig = plt.figure (figsize=(8,6), facecolor='white')
    ax1 = fig.add_axes ([0.06, 0.05, 0.90, 0.42])
    ax1.set_ylim (-1.5, 1.5)
    ax1.grid ()
    ax2 = fig.add_axes ([0.06, 0.53, 0.90, 0.42])
    ax2.set_xlim (5, fsamp / 2)
    ax2.set_ylim (-60, 15)
    ax2.set_xscale ('log')
    ax2.grid ()
    ax2.set_title (title)
    ax1.plot (Ain1 [0:500], color='b', lw=1)
    ax2.plot (Freq, 20 * np.log10 (np.abs (Spec) + 1e-10), color='b', lw=1)
    plt.show()
    
# Test all filters.    
#
print ("Close plot to start next test.")
run_test (M.FIL_NONE,   True,  "DC filter only")
run_test (M.FIL_ENB20K, True,  "DC filter and 20 kHz filter")
run_test (M.FIL_IEC_A,  False, "IEC A-filter")
run_test (M.FIL_IEC_C,  False, "IEC C-filter")
run_test (M.FIL_ITU468, False, "ITU468 filter")
run_test (M.FIL_DOLBY,  False, "Dolby filter")
print ("Done.")




# ----------------------------------------------------------------------------
#
#  Copyright (C) 2018-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


import globdef as G
from PyQt6 import QtGui, QtCore, QtWidgets
from matrixwin import *
from controls import *


class Delayelm (QtWidgets.QWidget):

    def __init__(self, parent, style, callb, index): 
        super (Delayelm, self).__init__(parent)
        self.resize (G.ELM_XS, G.ELM_YS)
        self.callb = callb
        self.index = index
        R = Rlinctl (self, style, 0, 100, 200, 10)
        R.move (7, 2)
        R.valueEvent.connect (self.value_event)
        R.show ()
        self.delayctl = R
        L = QtWidgets.QLabel (self)
        L.setFont (G.VAL_FONT)
        L.setText ('')
        L.setGeometry (0, 30, G.ELM_XS - 8, 14)
        L.setAlignment (QtCore.Qt.AlignmentFlag.AlignRight)
        self.delaytxt = L

    def redraw (self):
        delay = self.delayctl.get_value ();
        if self.delayctl.get_state () or (delay > 0):
            self.delaytxt.setText ('%5.1f' % (delay,))    
        else:
            self.delaytxt.setText ('')
        
    def value_event (self, args):
        self.redraw ()
        self.callb (self.index, G.ACT_DELAY, args.get_value (), 0)     
            
    def get_state (self):
        return self.delayctl.get_value ()

    def set_state (self, delay, act):        
        if delay is not None:
            self.delayctl.set_value (delay)
        self.delayctl.set_state (1 if act else 0)
        self.redraw ()

        

class Delaymatwin (Matrixwin):

    def __init__(self, parent, nrow, ncol, callb): 
        super (Delaymatwin, self).__init__(parent, nrow, ncol)
        pal = self.palette () 
        bgc = pal.color (pal.ColorRole.Window)
        style = RotaryStyle (11, bgc, [G.FG_N,G.COL5], [G.MK_N, G.COL0])
        self.controls = {}
        y = 1
        for r in range (nrow):
            x = 1
            for c in range (ncol):
                index = (r, c)
                E = Delayelm (self, style, callb, index)  
                E.move (x, y)
                self.controls [index] = E
                x += G.ELM_XS + 1
            y += G.ELM_YS + 1

    def get_state (self, row, col):
        return self.controls [(row, col)].get_value ()
    
    def set_state (self, row, col, delay, act):
        self.controls [(row, col)].set_state (delay, act)



# ----------------------------------------------------------------------------
#
#  Copyright (C) 2008-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


from zita_jacktools.jackclient import JackClient
from zita_jacktools import jackmatconv_ext
from zita_audiotools.audiofile import AudioFile
import numpy as np


class JackMatconv(JackClient) :

    """
    Jack client implementing a zero delay convolution matrix,
    using partitioned convolution with a single partition size
    equal to the Jack period.

    This version is optimized for a dense matrix of up to 100 x 100
    relatively short convolutions. It remains efficient for sparse
    matrices as unused matrix elements will take very little CPU
    time.

    The CPU load can be spread over multiple parallel threads.
    For longer or mixed size convolutions the JackConvolv class
    may provide better performance.
    """

    def __init__(self, size, ninp, nout, nthr, client_name, server_name = None):
        """
        Create a new JackMatconv instance, with 'ninp' inputs and 'nout'
        outputs, a maximum impulse length of 'size' samples, and using 
        up to 'nthr' threads.

        The result should be checked using get_state().
        The optional 'server_name' allows to select between running
        Jack servers.
        """
        self._ninp = ninp
        self._nout = nout
        self._jmatconv, base = jackmatconv_ext.makecaps (self, client_name, server_name, size, ninp, nout, nthr)
        super().__init__(base)
        

    def silence(self):
        """
        Set the state to SILENCE.

        In this state the convolver outputs silence. This is
        also the initial state unless the constructor failed.
        """
        return jackmatconv_ext.set_state (self._jmatconv, JackClient.SILENCE)


    def process(self):
        """
        Set the state to PROCESS.

        In this state the convolver is operating normally.
        It is still possible to modify the impulse responses.
        """
        return jackmatconv_ext.set_state (self._jmatconv, JackClient.PROCESS)


    def load_impulse(self, data, inp, out, gain = 1.0):
        """
        Load the impulse response for a single input,output pair.

        The 'data' must be a single dimension float32 numpy array
        or array view, or 'None' to clear the current response.
        The 'gain' parameter is a linear gain factor.
        """
        return jackmatconv_ext.load_impulse (self._jmatconv, data, inp, out, gain)
        

    def load_matrix (self, file, size, gain = 1.0):
        """
        Convenience function, load the entire convolution
        matrix from an audio file.

        If 'ninp' and 'nout' are the matrix dimensions, the
        file must have 'ninp' channels and consist of 'nout'
        sections of 'size' frames each. 
        The 'gain' parameter is a linear gain factor.
        """
        jname, fsamp, psize = self.get_jack_info ()
        F = AudioFile ()
        try:
            F.open_read (file)
            assert F.channels () == self._ninp
            assert F.filesize () == self._nout * size
            A = np.empty ((size, self._ninp), dtype = np.float32)
            for i in range (self._nout):
                F.read (A)
                for j in range (self._ninp):
                     self.load_impulse (A [:,j], j, i, gain)
        finally:             
            F.close ()
    

    def load_matrix_transposed (self, file, size, gain = 1.0):
        """
        Convenience function, load the entire convolution
        matrix from an audio file.

        If 'ninp' and 'nout' are the matrix dimensions, the
        file must have 'nout' channels and consist of 'ninp'
        sections of 'size' frames each. 
        The 'gain' parameter is a linear gain factor.
        """
        jname, fsamp, psize = self.get_jack_info ()
        F = AudioFile ()
        try:
            F.open_read (file)
            assert F.channels () == self._nout
            assert F.filesize () == self._ninp * size
            A = np.empty ((size, self._nout), dtype = np.float32)
            for i in range (self._ninp):
                F.read (A)
                for j in range (self._nout):
                     self.load_impulse (A [:,j], i, j, gain)
        finally:             
            F.close ()
    



# ----------------------------------------------------------------------------
#
#  Copyright (C) 2012-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


from zita_jacktools.jackclient import JackClient
from zita_jacktools import jackconvolv_ext


class JackConvolv(JackClient) :

    """
    Convolution matrix using the zita-convolver library.
    """

    def __init__(self, ninp, nout, client_name, server_name = None):
        """
        Create a new JackConvolv instance, with 'ninp' inputs and 'nout'
        outputs. 

        The result should be checked using get_state().
        The optional 'server_name' allows to select between running
        Jack servers.
        """
        self._ninp = ninp
        self._nout = nout
        self._jconvolv, base = jackconvolv_ext.makecaps (self, client_name, server_name, ninp, nout)
        super().__init__(base)
        

    def silence(self):
        """
        Set the state to SILENCE. 

        In this state the convolver outputs silence. This is
        also the initial state unless the constructor failed.
        """
        return jackconvolv_ext.set_state (self._jconvolv, JackClient.SILENCE)


    def process(self):
        """
        Set the state to PROCESS.

        In this state the convolver is operating normally.
        It is still possible to modify the impulse responses.
        """
        return jackconvolv_ext.set_state (self._jconvolv, JackClient.PROCESS)

    
    def configure(self, maxlen, density):
        """
        Configure or reconfigure the convolver. 
        The object must be in the SILENCE state.
        """
        return jackconvolv_ext.configure (self._jconvolv, maxlen, density)


    def cleanup(self):
        """
        Reset the concolver configuration to the initial empty state.
        The object must be in the SILENCE state.
        """
        return jackconvolv_ext.configure (self._jconvolv, 0, 0.0)

    
    def impdata_create(self, data, inp, out, offs = 0):
        """
        Add 'data' to the impulse response for the given input and
        output, starting at sample postion 'offs'.

        The object must be in SILENCE state and configured. The 'data'
        argument should be a single dimension float32 numpy array or
        array view. 
        """
        return  jackconvolv_ext.impdata_write (self._jconvolv, data, inp, out, offs, 1)
        

    def impdata_link(self, inp1, out1, inp2, out2):
        """
        Set the impulse response for (inp2, out2) to a copy of the
        one for (inp1, out1). 

        The object must be in SILENCE state and configured.
        Links can be used to conserve memory when a number of identical
        long impulse responses is required. The precomputed frequency
        domain form of the impulse response is shared between linked
        matrix elements.
        """
        return jackconvolv_ext.impdata_link (self._jconvolv, inp1, out1, inp2, out2)
        

    def impdata_clear(self, data, inp, out):
        """
        Clear the existing impulse response for the given input and
        output.

        This can be called while in PROCESS state and is typically
        used before impdata_update() to replace the existing impulse
        responses.
        """
        return jackconvolv_ext.impdata_write (self._jconvolv, None, inp, out, 0, 0)
        

    def impdata_update(self, data, inp, out, offs = 0):
        """
        Add 'data' to the impulse response for the given input and
        output, starting at sample postion 'offs'.

        This method is used to modify existing impulse response data
        while in PROCESS state. It will modify existing partitions only.
        The 'data' argument should be a single dimension float32 numpy
        array or array view. 
        """
        return jackconvolv_ext.impdata_write (self._jconvolv, data, inp, out, offs, 0)
        





# ----------------------------------------------------------------------------
#
#  Copyright (C) 2018-2022 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


import sys
import signal
import globdef as G
from PyQt6 import QtGui, QtCore, QtWidgets
from copy import deepcopy 
from os.path import *
from buttonbase import *
from gainmatwin import *
from delaymatwin import *
from inputwin import *
from outputwin import *
from matrixstate import *
from zita_jacktools.jackmatrix import db2lin


class Mainwindow (QtWidgets.QWidget):


    loadreqEvent = QtCore.pyqtSignal(object)

    
    def __init__(self, inps, outs, matrix, label = 'Matrix'): 
        super (Mainwindow, self).__init__()
        
        # Init main window.
        self.setWindowTitle (label)
        self.ninp = len (inps)
        self.nout = len (outs)
        self.move (100, 100)
        xs = self.ninp * (G.ELM_XS + 1) + G.OP_XS + 10
        ys = self.nout * (G.ELM_YS + 1) + G.IP_YS + 60
        if xs < 200: xs = 200
        self.setFixedSize (xs, ys)

        # Button styles.
        pal = self.palette () 
        bgc = pal.color (pal.ColorRole.Window)
        bsty1 = ButtonStyle (49, 19, bgc, [G.FG_N,G.COL4,G.COL5], [G.MK_N,G.COL0,G.COL0], G.BUT_FONT, 10)
        bsty2 = ButtonStyle (28, 21, bgc, [G.FG_N,G.COL5,G.COL2], [G.MK_N,G.COL0,G.COL9], G.BUT_FONT, 10)

        # Create the top menu buttons.
        B = ButtonBase (self, bsty1, True, 0)
        B.set_text ("Load")
        B.move (0, 0)
        B.brelseEvent.connect (self.load_action)
        B = ButtonBase (self, bsty1, True, 1)
        B.set_text ("Save")
        B.move (50, 0)
        B.brelseEvent.connect (self.save_action)
        B = ButtonBase (self, bsty1, True, 1)
        B.set_text ("SaveAs")
        B.move (100, 0)
        B.brelseEvent.connect (self.saveas_action)
        B = ButtonBase (self, bsty1, True, 0)
        B.set_text ("Reset")
        B.move (150, 0)
        B.brelseEvent.connect (self.clear_action)

        # Create the matrix windows.
        x = 10
        y = 20
        self.inputwin = Inputwin (self, self.ninp, self.input_callb, inps) 
        self.inputwin.move (x, y)
        y += G.IP_YS
        self.gainmatwin = Gainmatwin (self, self.nout, self.ninp, self.matrix_callb)
        self.gainmatwin.move (x, y)
        self.delaymatwin = Delaymatwin (self, self.nout, self.ninp, self.matrix_callb)
        self.delaymatwin.hide ()
        self.delaymatwin.move (x, y)
        x += self.gainmatwin.xs
        self.outputwin = Outputwin (self, self.nout, self.output_callb, outs) 
        self.outputwin.move (x, y)
        y -= G.IP_YS
        self.bgain = ButtonBase (self, bsty1, True, 0)
        self.bgain.set_text ("Gain")
        self.bgain.move (x + 20, y + 20)
        self.bgain.set_state (1)
        self.bgain.bpressEvent.connect (self.select_gain)
        self.bdelay = ButtonBase (self, bsty1, True, 0)
        self.bdelay.set_text ("Delay")
        self.bdelay.move (x + 20, y + 40)
        self.bdelay.bpressEvent.connect (self.select_delay)

        # Create preset buttons.
        n = 2 * min (10, (xs - 20) // 60)
        self.bpreset = [ ButtonBase (self, bsty2, True, i) for i in range (n) ]
        self.presets = [ Matrixstate (self.ninp, self.nout) for i in range (n) ]
        self.npreset = n;
        self.curpres = None
        for i in range (n):
            self.bpreset [i].move (10 + i * 30, ys - 30)
            self.bpreset [i].set_text (str (i+1))
            self.bpreset [i].brelseEvent.connect (self.preset_action)
        
        # Rename the JackMatrix ports.
        self.jackmatrix = matrix
        for i in range (self.ninp):
            self.jackmatrix.rename_input (i, "in-" + inps [i])
        for i in range (self.nout):
            self.jackmatrix.rename_output (i, "out-" + outs [i])

        # Apply current state.    
        self.currstate = Matrixstate (self.ninp, self.nout)
        self.apply_iostate ()
        self.apply_matstate ()
        self.path = '.'
        self.name = 'presets'
        self.loadreqEvent.connect (self.loadEvent)
        self.loadind = None
        self.show ()

        
    def select_gain (self, args):
        # Select gain matrix.
        self.bgain.set_state (1)
        self.bdelay.set_state (0)
        self.gainmatwin.show ()
        self.delaymatwin.hide ()

        
    def select_delay (self, args):
        # Select delay matrix.
        self.bdelay.set_state (2)
        self.bgain.set_state (0)
        self.delaymatwin.show ()
        self.gainmatwin.hide ()

        
    def input_callb (self, index, action, value, button):
        # Callback from input controls.
        W = self.inputwin
        D = self.currstate.inpgain
        E = D.elem (index)
        if   action == G.ACT_GAIN: E.gain = value
        elif action == G.ACT_MUTE: E.mute = value
        elif action == G.ACT_SOLO:
            if value and button == 1:
                W.exc_solo (index)
        else: return
        s = W.any_solo ()
        if action == G.ACT_SOLO:
            for i in range (0, self.ninp):
                self.set_inpgain (i, s)
        else:
            self.set_inpgain (index, s)
            self.preset_modif ()

            
    def output_callb (self, index, action, value, button):
        # Callback from output controls.
        W = self.outputwin
        D = self.currstate.outgain
        E = D.elem (index)
        if   action == G.ACT_GAIN: E.gain = value
        elif action == G.ACT_MUTE: E.mute = value
        elif action == G.ACT_SOLO:
            if value and button == 1:
                W.exc_solo (index)
        else: return
        s = W.any_solo ()
        if action == G.ACT_SOLO:
            for i in range (0, self.nout):
                self.set_outgain (i, s)
        else:
            self.set_outgain (index, s)
            self.preset_modif ()

        
    def matrix_callb (self, index, action, value, flags):
        # Callback from matrix controls.
        inp = index [1]
        out = index [0]
        if action == G.ACT_GAIN:
            E = self.currstate.matrix.elem (inp, out)
            E.gain = value
            E.inv = flags
            g = db2lin (value)
            if E.inv: g *= -1
            self.jackmatrix.set_matrix_gain (inp, out, g)
            self.delaymatwin.set_state (out, inp, None, E.gain > -200)
        elif action == G.ACT_DELAY:
            E = self.currstate.matrix.elem (inp, out)
            E.delay = value
            self.jackmatrix.set_matrix_delay (inp, out, 1e-3 * value)
        else: return
        self.preset_modif ()

        
    def load_action (self):    
        # Load presets from file.
        name = QtWidgets.QFileDialog.getOpenFileName (self, 'Load preset', self.path, '')
        if len (name [0]) == 0: return
        self.name = name [0]
        self.path = dirname (self.name)
        with open (name [0], 'r') as fobj:
            self.preset_modif ()
            while True:
                s = fobj.readline ()
                if len (s) == 0: break
                if s [0] == '#': continue
                s = s.split ()
                if len (s) != 2: continue
                if s [0] == 'PRE':
                    k = int (s [1]) 
                    if k > self.npreset: break
                    self.presets [k-1].read (fobj)
        if self.curpres is not None:
            self.curpres.set_state (0)
            self.curpres = None

            
    def save_action (self):                
        # Save all presets to file.
        if self.name is not None:
            self.save_presets ()

            
    def saveas_action (self):                
        # Save all presets to file.
        name = QtWidgets.QFileDialog.getSaveFileName (self, 'Save preset', self.path, '')
        if len (name [0]) == 0: return
        self.name = name [0]
        self.path = dirname (self.name)
        self.save_presets ()

        
    def save_presets (self):                
        # Save all presets to file.
        with open (self.name, 'w') as fobj:
            for k in range (self.npreset):
                fobj.write ("# --------------\n")
                fobj.write ("PRE %d\n" % (k+1,))
                self.presets [k].write (fobj)            

                
    def clear_action (self, B):
        # Clear current state or only solo.
        if B.modifiers == QtCore.Qt.KeyboardModifier.ShiftModifier:
            self.currstate.reset ()
            self.apply_matstate ()
            if self.curpres is not None:
                self.curpres.set_state (0)
                self.curpres = None
        self.inputwin.exc_solo (None)
        self.outputwin.exc_solo (None)
        self.apply_iostate ()
      
            
    def preset_action (self, B):
        # Load or store a preset.
        if B.modifiers == QtCore.Qt.KeyboardModifier.ShiftModifier:
            self.presets [B.index] = deepcopy (self.currstate)
        else:
            self.currstate = deepcopy (self.presets [B.index])
            self.apply_iostate ()
            self.apply_matstate ()
        if self.curpres is not None:
            self.curpres.set_state (0)
        self.curpres = B
        self.curpres.set_state (1)

        
    def preset_modif (self):
        # Mark preset as modified
        if self.curpres is not None:
            self.curpres.set_state (2)

            
    def apply_iostate (self):
        # Copy current state to JackMatrix and GUI.
        # Inputs.
        D = self.currstate.inpgain
        s = self.inputwin.any_solo ()
        for inp in range (self.ninp):
            E = D.elem (inp)
            self.set_inpgain (inp, s)
            self.inputwin.set_state (inp, E.gain, E.mute)
        # Outputs.
        D = self.currstate.outgain
        s = self.outputwin.any_solo ()
        for out in range (self.nout):
            E = D.elem (out)
            self.set_outgain (out, s)
            self.outputwin.set_state (out, E.gain, E.mute)

            
    def apply_matstate (self):
        # Copy current state to JackMatrix and GUI.
        # Matrix.
        D = self.currstate.matrix
        for out in range (self.nout):
            for inp in range (self.ninp):
                E = D.elem (inp, out)
                g = db2lin (E.gain)
                if E.inv: g *= -1
                self.jackmatrix.set_matrix_gain (inp, out, g)
                self.jackmatrix.set_matrix_delay (inp, out, 1e-3 * E.delay)
                self.gainmatwin.set_state (out, inp, E.gain, E.inv)
                self.delaymatwin.set_state (out, inp, E.delay, E.gain > -200)

                
    def set_inpgain (self, inp, anysolo):
        # Set input gain including solo logic.
        W = self.inputwin
        E = self.currstate.inpgain.elem (inp)
        if E.mute or (anysolo and not W.get_solo (inp)): g = 0
        else: g = db2lin (E.gain)
        self.jackmatrix.set_input_gain (inp, g)

        
    def set_outgain (self, out, anysolo):
        # Set output gain including solo logic.
        W = self.outputwin
        E = self.currstate.outgain.elem (out)
        if E.mute or (anysolo and not W.get_solo (out)): g = 0
        else: g = db2lin (E.gain)
        self.jackmatrix.set_output_gain (out, g)

        
    def loadEvent (self, E):
        ButtonBase.modifiers = None
        self.preset_action (self.bpreset [self.loadind])

        
    def loadreq (self, k):
        self.loadind = k
        self.loadreqEvent.emit (self)
